//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2021
//#
//#
//# File Name:  EPT_4CE6_AF_D1_Top.v
//# Author:     Earth People Technology
//# Date:       March 14, 2021
//# Revision:   A
//#
//# Development: EPT Platform Demonstration Project 
//# Application: Altera Cyclone IV FPGA
//# Description: This file contains verilog code which will allow access
//#              to Active Transfer Library. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			3/14/21 	1			Created
//#
//#									
//#
//#######################################################################


//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_4CE6_AF_D1_Top (

	
	input  wire                CLK_66MHZ,
	input  wire                RST,

	//UART COMMAND BUS
	output wire                UART_OUT, //To external 
    input wire                 UART_IN, //From external
  
	
	output wire   [7:0]        XIO_1,       //XIO -- D2-D9
	output wire   [5:0]        XIO_2,       //XIO -- D10-D12
	output wire   [5:0]        XIO_3,       //XIO -- D22-D29
	input wire    [5:0]        XIO_4,       //XIO -- D30-D37
	input wire   [5:0]        XIO_5,       //XIO -- D38-D45
	output wire   [3:0]        XIO_6_OUT,   //XIO -- D46-D53
	input wire    [31:4]       XIO_6,       //XIO -- D46-D53
	output wire   [2:0]        XIO_7       //XIO -- D69,D70,D71,D74,D75,D76
	
	);

   //-----------------------------------------------
   // Parameters
   //-----------------------------------------------
   
   parameter                   GLOBAL_RESET_COUNT = 12'h09c8;

   //LED State Machine Parameters
   parameter                   IDLE = 0,
	                           SELECT_MODE = 1,
                               LOAD_REGISTER = 2,
                               LOAD_LEDS = 3,
                               WAIT_FOR_TIMER = 6;

   //Control Register selections for the LED Blinky
   parameter                   RANDOM_NUMBER = 4'h1;
   parameter                   SHIFT_LEFT = 4'h2;
   parameter                   SHIFT_RIGHT = 4'h3;
   parameter                   COUNT_UP = 4'h4;
   parameter                   COUNT_DOWN = 4'h5;
   parameter                   STATIC_VALUE = 6'h6;
   
   //Timer Value Low Limit
   parameter                   TIMER_LOW_LIMIT = 24'h3ff;

`ifdef SIM
   	reg [8*30:1] state_name;
`endif
   
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************
    wire                       CLK_66;
	wire                       RST_N;
	wire [1:0]                 aa;

	wire [23:0]                UC_IN;
	wire [21:0]                UC_OUT;

	//Finite State Machine control registers
	reg 	[6:0]              state, next;

	//Trigger Signals
	reg [7:0]                  trigger_out;
	wire [7:0]                 trigger_in_byte;	
	reg [7:0]                  trigger_from_host_latch;
	
    //UART_IN meta registers
	reg                        uart_meta;
	reg                        uart_mid;
	reg                        uart_txd;

	//LED registers
	wire                        led_reset;
	
	//Switch registers
	wire                        switch_reset;
	
	//Transfer registers
	wire                       transfer_out;
	reg                        transfer_out_reg;
	wire                       transfer_in_received;
	wire [7:0]                 transfer_in_byte;
	wire  [7:0]                 transfer_out_byte;
	reg  [3:0]                 transfer_to_host_counter;
	reg  [3:0]                 transfer_to_host_state;

    //Transfer Control registers
	wire                       transfer_in_loop_back;
	reg                        transfer_in_received_reg;
	reg  [3:0]                 transfer_control_state;
	reg  [7:0]                 transfer_control_byte;
	wire   [7:0]               control_register;
	
	
	//Block Transfer Out registers
	reg [2:0]                  state_block_loopback;
	reg                        block_out_reg;
	wire                       block_out_received;
	reg [7:0]                  block_out_counter;
	reg                        start_block_transfer;
	reg                        start_block_transfer_reg;
	wire                       block_byte_ready;
	wire [7:0]                 block_out_byte;
	reg [1:0]                  block_transfer_out_state;
	
	//Block Transfer In registers
	wire [7:0]                 block_in_data;
	wire                       block_in_rcv;
	wire                       block_in_loopback;
	wire  [7:0]                ept_length;
	wire  [7:0]                uc_length;
	wire                       block_busy;
	reg   [7:0]                block_in_data_counter;
	reg                        fifo_wren_reg;

	//Reset signals
	wire                       reset;
	reg [11:0]                 reset_counter;
	reg                        reset_signal_reg;
	
	//LED Blinky Transfer registers
	reg                        led_start_transfer;
	reg                        led_start_transfer_reg;
	wire                       led_transfer_in_received;
	reg [7:0]                  led_host_transfer_byte;
	wire [7:0]                 led_device_transfer_byte;

	//LED BLinky Signals
	reg [23:0]                 timer_value;	
	reg [7:0]                  seed_value;
	reg [23:0]                 led_delay_counter;
	reg [3:0]                  random_number;
	reg [3:0]                  random_num_counter;
	reg [3:0]                  led_reg;
	wire                       random_number_generated;
	reg                        start_blinky;
	reg [3:0]                  static_load_value;
	
	reg [3:0]                  shift_count_value;

	//Arduino Data Collector Interface
	wire                        LEDExt;
	wire                        c_enable;
	wire [7:0]                  data_from_arduino;
	
    //Transfer Write from Arduino
	reg                        transfer_write_reg;
	reg                        transfer_write;
	reg  [7:0]                 transfer_write_byte;

	//Control Registers Output Signals
	wire                       start_stop_cntrl;

	//Registers for Test
	wire  [3:0]                active_serial_state_out;
	wire  [11:0]                active_transfer_test_out;



//***************************************************************************
//* 	Signal Assignments	
//***************************************************************************
   
   //Clock and Reset
   assign            aa[1] = CLK_66MHZ;
   assign            aa[0] = RST_N;
   assign            RST_N = reset_signal_reg;
    
    //Control Register Map
   assign            start_stop_cntrl = control_register[0];
   //assign            load_block_image      = control_register[1];
   //assign            load_ept_image        = control_register[2];
   //assign            load_face_1_image     = control_register[3];
   
   //Arduino Connections
   assign            LEDExt = XIO_5[5];                       //XIO_5 -- UB59 -- D15 
   assign            XIO_1[3] = start_stop_cntrl;             //XIO_1 -- UB3 -- D11        
   assign            c_enable = XIO_5[3];                     //XIO_5 -- UB57 -- D17
   assign            data_from_arduino[7] = XIO_6[18];         //XIO_6 -- UB40 -- D40
   assign            data_from_arduino[6] = XIO_6[17];         //XIO_6 -- UB39 -- D39
   assign            data_from_arduino[5] = XIO_6[16];         //XIO_6 -- UB38 -- D38
   assign            data_from_arduino[4] = XIO_6[15];         //XIO_6 -- UB37 -- D37
   assign            data_from_arduino[3] = XIO_6[14];         //XIO_6 -- UB36 -- D36
   assign            data_from_arduino[2] = XIO_6[13];         //XIO_6 -- UB35 -- D35
   assign            data_from_arduino[1] = XIO_6[12];         //XIO_6 -- UB34 -- D34
   assign            data_from_arduino[0] = XIO_6[11];         //XIO_6 -- UB33 -- D33
 
    //Transfer registers
   assign            transfer_out = transfer_write_reg | transfer_write;
   assign            transfer_out_byte = transfer_write_byte;
   
   //LEDs
   assign            XIO_6_OUT[0] = ~start_stop_cntrl;
   assign            XIO_6_OUT[1] = ~c_enable;
   assign            XIO_6_OUT[2] = ~LEDExt;
   assign            XIO_6_OUT[3] = ~transfer_write_reg;

   assign           XIO_1[0] = UART_OUT; 
   assign           XIO_1[1] = active_serial_state_out[1]; 
   assign           XIO_1[2] = active_serial_state_out[2]; 
   //assign           XIO_1[3] = active_serial_state_out[3]; 
   assign           XIO_1[4] = active_transfer_test_out[0];
   assign           XIO_1[5] = active_transfer_test_out[1];
   assign           XIO_1[6] = active_transfer_test_out[2];
   assign           XIO_1[7] = active_transfer_test_out[3];

   assign           XIO_3[0] = c_enable; 
   assign           XIO_3[1] = transfer_write_reg; 
   assign           XIO_3[2] = transfer_write; 
   assign           XIO_3[3] = transfer_out; 
   assign           XIO_3[4] = transfer_in_received; 
   assign           XIO_3[5] = active_transfer_test_out[9]; 
   
   //assign           XIO_5[0] = active_transfer_test_out[10]; 
   //assign           XIO_5[1] = active_transfer_test_out[11]; 
   //assign           XIO_5[2] = 1'b0; 
   //assign           XIO_5[3] = 1'b0; 
   //assign           XIO_5[4] = 1'b0; 
   //assign           XIO_5[5] = 1'b0; 

  
//**************************************************
//* 	Reset Signal	
//**************************************************
	always @(posedge CLK_66MHZ or negedge RST)
	begin
	  if(!RST)
	  begin
			reset_signal_reg <= 1'b0;
			reset_counter <= 0;
	  end
	  else
	  begin
		if( reset_counter < GLOBAL_RESET_COUNT )
		begin
			reset_signal_reg <= 1'b0;
			reset_counter <= reset_counter + 1'b1;
		end
		else
		begin
			reset_signal_reg <= 1'b1;
		end
	  end
	end

   //-----------------------------------------------
   // Register the incoming UART signal
   // 
   //-----------------------------------------------

  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
     if(!RST_N)
	 begin
	     uart_meta <= 1'b0;
		 uart_mid <= 1'b0;
		 uart_txd <= 1'b0;
	end
	else
	 begin
	     uart_meta <= UART_IN;
		 uart_mid <= uart_meta;
		 uart_txd <= uart_mid;
	 end
  
  end


   //-----------------------------------------------
   // Detect Transfer From Arduino
   //-----------------------------------------------
    always @(posedge CLK_66MHZ or negedge RST) 
	begin
       if (!RST)
       begin
			transfer_write_reg <= 1'b0;
			transfer_write <= 1'b0;
			transfer_write_byte <= 0;
       end 
       else 
       begin 
	        if(c_enable & !transfer_write_reg)
			begin
			    transfer_write_reg <= 1'b1;
				transfer_write <= 1'b1;
				transfer_write_byte <= data_from_arduino;
			end
	        else if(c_enable & transfer_write_reg)
			begin
			    transfer_write_reg <= 1'b1;
				transfer_write <= 1'b0;
			end
	        else if(!c_enable & transfer_write_reg)
			begin
			    transfer_write_reg <= 1'b0;
				transfer_write <= 1'b0;
			    transfer_write_byte <= 0;
			end
			else
			begin
			end
	   end
    end


   //-----------------------------------------------
   // State Machine: Control Register from Transfer In 
   //-----------------------------------------------
  /*
     Control Register Register Map
	 
	 control_register[0]  =  transfer_in_loop_back
	 control_register[1]  =  load_block_image//block_in_loopback 
	 control_register[2]  =  load_ept_image
	 control_register[3]  =  load_face_1_image
	 control_register[4]  =  LED Select Mode[0]
	 control_register[5]  =  LED Select Mode[1]
	 control_register[6]  =  LED Select Mode[2]
	 control_register[7]  =  LED Select Mode[3]
	 

  */
  active_control_register      ACTIVE_CONTROL_REG_INST
  (
   .CLK                        (CLK_66MHZ),
   .RST                        (RST_N),
   .TRANSFER_IN_RECEIVED       (transfer_in_received),
   .TRANSFER_IN_BYTE           (transfer_in_byte),

   .CONTROL_REGISTER           (control_register)
   );
   
   //-----------------------------------------------
   // Instantiate the EPT Library
   //-----------------------------------------------

	active_serial_library	   ACTIVE_SERIAL_LIBRARY_INST
	(	
	.aa                        (aa),
	.UART_OUT                  (UART_OUT),
	.UART_IN                   (uart_txd),

	.UC_IN                     (UC_IN),
	.UC_OUT                    (UC_OUT),
	
	.STATE_OUT                 (active_serial_state_out),
	.TEST_OUT                  (active_transfer_test_out) 
	
	);

   //-----------------------------------------------
   // Instantiate the EPT Library
   //-----------------------------------------------
wire [22*4-1:0]  uc_out_m;
eptWireOR # (.N(4)) wireOR (UC_OUT, uc_out_m);

/*
    Active Trigger Register Mapping
	
	trigger_in_byte[0]  =  Load Static Value
	trigger_in_byte[1]  =  Load shift_count_value
	trigger_in_byte[2]  =  
	trigger_in_byte[3]  =  
	trigger_in_byte[4]  =  Load Timer Value, timer_value[15:8]
	trigger_in_byte[5]  =  Load Timer Value, timer_value[23:16]
	trigger_in_byte[6]  =  Load linear_feedback_shift_register
	trigger_in_byte[7]  =  start_block_transfer = 1'b1
	

*/
	active_trigger             ACTIVE_TRIGGER_INST
	(
	 .uc_clk                   (CLK_66MHZ),
	 .uc_reset                 (RST_N),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 0*22 +: 22 ]),

	 .trigger_to_host          (trigger_out),
	 .trigger_to_device        (trigger_in_byte)
	
	);

    active_transfer            ACTIVE_TRANSFER_INST_1
	(
	 .uc_clk                   (CLK_66MHZ),
	 .uc_reset                 (RST_N),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 1*22 +: 22 ]),
	
	 .start_transfer           (led_start_transfer),
	 .transfer_received        (led_transfer_in_received),
	 
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h1),

	 .transfer_to_host         (led_host_transfer_byte),
	 .transfer_to_device       (led_device_transfer_byte)	
    );
	
	active_transfer            ACTIVE_TRANSFER_INST_2
	(
	 .uc_clk                   (CLK_66MHZ),
	 .uc_reset                 (RST_N),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 2*22 +: 22 ]),
	
	 .start_transfer           (transfer_out),
	 .transfer_received        (transfer_in_received),
	
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h2),

	 .transfer_to_host         (transfer_out_byte),
	 .transfer_to_device       (transfer_in_byte)	
	);
	
	active_block               BLOCK_TRANSFER_INST
	(
	 .uc_clk                   (CLK_66MHZ),
	 .uc_reset                 (RST_N),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 3*22 +: 22 ]),
	
	 .start_transfer           (block_out_reg),
	 .transfer_received        (block_in_rcv),
	 
	 .transfer_ready           (block_byte_ready),
	 .transfer_busy            (block_busy),

	 .ept_length               (ept_length),
	
	 .uc_addr                  (3'h4),
	 .uc_length                (ept_length),

	 .transfer_to_host         (block_out_byte),
	 .transfer_to_device       (block_in_data)
	
	);


	
endmodule

