//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2021
//#
//#
//# File Name:  EPT_4CE6_AF_D1_Top.v
//# Author:     Earth People Technology
//# Date:       March 14, 2021
//# Revision:   A
//#
//# Development: EPT Platform Demonstration Project 
//# Application: Altera Cyclone IV FPGA
//# Description: This file contains verilog code which will allow access
//#              to Active Transfer Library. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			3/14/21 	1			Created
//#			3/12/24 	1			Changed Block Out to Logic Analyzer
//#
//#									
//#
//#######################################################################


//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_4CE6_AF_D1_Top (

	
	input  wire                CLK_66MHZ,
	input  wire                RST,

	//UART COMMAND BUS
	output wire                UART_OUT, //To external 
    input wire                 UART_IN, //From external
  
	
	input  wire   [7:0]        XIO_1,       //XIO -- UB
	//input  wire   [5:0]        XIO_2,       //XIO -- UB
	input  wire   [5:0]        XIO_3,       //XIO -- UB
	input  wire   [5:0]        XIO_4,       //XIO -- UB
	output wire   [5:0]        XIO_5,       //XIO -- UB
	inout  wire   [31:0]       XIO_6,       //XIO -- UB
	output wire   [3:0]        XIO_7,       //XIO -- UB
	
	input wire                 PB_SWITCH_1,
	input wire                 PB_SWITCH_2,
	
	input wire                 SLIDE_SWITCH_1,
	input wire                 SLIDE_SWITCH_2	
	
	
	);

   //-----------------------------------------------
   // Parameters
   //-----------------------------------------------
   
   parameter                   GLOBAL_RESET_COUNT = 12'h09c8;
   
   //Transfer to Host State Machine
   parameter                   TRANSFER_HOST_IDLE = 1,
                               TRANSFER_HOST_LOOPBACK_START = 2,
							   TRANSFER_HOST_LOOPBACK_COMPLETE = 3;

    // Block Transfer Options
	parameter                  BLOCK_COUNT_16       = 8'h10;

	//Block Transfer to Host State Machine
   parameter                   BLOCK_LOOPBACK_IDLE = 1,
                               BLOCK_LOOPBACK_INIT = 2,
							   FIFO_EN_HIGH = 3,
							   FIFO_EN_LOW = 4;

	//Number of GPIO Storage Registers on the MegaProLogic
   parameter                   DUE_GPIO_MAX_NUMBER = 8;	

	//Block Transfer to Host Counter
   parameter                   BLK_TRANSFER_OUT_IDLE = 1,
			                   BLK_TRANSFER_OUT_COUNT = 2;

   //LED State Machine Parameters
   parameter                   IDLE                  = 0,
                               LOAD_IMAGE            = 1,
	                           SELECT_MODE           = 2,
							   UPDATE_IMAGE          = 3,
							   WAIT_FOR_TIMER_HIGH_Z = 4,
                               LOAD_LED_ARRAY        = 5,
                               WAIT_FOR_TIMER_ACTIVE = 6,
                               CHECK_FOR_NEW_IMAGE   = 7;
							   
  //Block In for Image Transfer In
   parameter                   BLOCK_IMAGE_IDLE           = 0,
                               BLOCK_IMAGE_WAIT_FOR_READY = 1,
                               BLOCK_IMAGE_STORE_BYTE     = 2,
                               BLOCK_IMAGE_CHECK_COMPLETE = 3; 
							   
  //Slide Switch Block
   parameter                   BLOCK_SLIDE_SWITCH_LENGTH = 2;
   parameter                   BLK_SLIDE_SWITCH_IDLE     = 0,
	                           BLK_SLIDE_SWITCH_COUNT    = 1;


   //Control Register selections for the LED Blinky
   parameter                   RANDOM_NUMBER         = 4'h1;
   parameter                   SHIFT_LEFT            = 4'h2;
   parameter                   SHIFT_RIGHT           = 4'h3;
   parameter                   SCROLL_UP             = 4'h4;
   parameter                   SCROLL_DOWN           = 4'h5;
   parameter                   STATIC_VALUE          = 4'h6;
   parameter                   STATIC_IMAGE          = 4'h7;
   parameter                   BLINK_IMAGE           = 4'h8;
   parameter                   SHIFT_IMAGE_RIGHT     = 4'h9;
   parameter                   SHIFT_IMAGE_LEFT      = 4'ha;  
   parameter                   SHIFT_IMAGE_DOWN      = 4'hb;
   parameter                   SHIFT_IMAGE_UP        = 4'hc;
   
   //Push Button Switches
   parameter                   PB_SWITCH_1_ASSERT    = 8'h04;
   parameter                   PB_SWITCH_2_ASSERT    = 8'h08;

   //Timer Value Low Limit
`ifdef SIM
   parameter                   TIMER_LOW_LIMIT       = 24'h1f;
   parameter                   HIGH_Z_TIMER_VALUE    = 24'h8f;
`else
   parameter                   TIMER_LOW_LIMIT       = 24'h325aa0;
   parameter                   HIGH_Z_TIMER_VALUE    = 24'ha1220; //10 ms
   //parameter                   HIGH_Z_TIMER_VALUE    = 24'h325aa0; //50 ms
`endif

`ifdef SIM
   	reg [8*30:1] state_name;
`endif
   
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************
    wire                       CLK_66;
	wire                       RST_N;
	wire [1:0]                 aa;

	wire [23:0]                UC_IN;
	wire [21:0]                UC_OUT;

	//Finite State Machine control registers
	reg  [7:0]                 state, next;

	//Trigger Signals
	reg  [7:0]                 trigger_out;
	wire [7:0]                 trigger_in_byte;	
	reg  [7:0]                 trigger_from_host_latch;
	
    //UART_IN meta registers
	reg                        uart_meta;
	reg                        uart_mid;
	reg                        uart_txd;

    //LED registers
	wire                        led_reset;
	
	//Switch registers
	wire                        switch_reset;
	
	//Transfer registers
	reg                        transfer_out_reg;
	wire                       transfer_in_received;
	wire [7:0]                 transfer_in_byte;
	reg  [7:0]                 transfer_out_byte;
	reg  [3:0]                 transfer_to_host_counter;
	reg  [3:0]                 transfer_to_host_state;

    //Transfer Control registers
	reg                        control_register_out_reg;
	wire                       control_register_in_received;
	wire [7:0]                 control_register_in_byte;
	reg  [7:0]                 control_register_out_byte;
	wire                       transfer_in_loop_back;
	reg                        transfer_in_received_reg;
	reg   [3:0]                transfer_control_state;
	reg   [7:0]                transfer_control_byte;
	wire  [7:0]                control_register;
	
	//Block Transfer Out registers
	reg  [2:0]                 state_block_loopback;
	reg                        block_out_reg;
	wire                       block_out_received;
	reg  [7:0]                 block_out_counter;
	reg                        start_block_transfer;
	reg                        start_block_transfer_reg;
	wire                       block_byte_ready;
	reg  [7:0]                 block_out_byte;
	reg  [1:0]                 block_transfer_out_state;
	
	//Block Transfer In registers
	wire [7:0]                 block_in_data;
	wire                       block_in_rcv;
	wire                       block_in_loopback;
	wire  [7:0]                ept_length;
	wire  [7:0]                uc_length;
	wire                       block_busy;
	reg   [7:0]                block_in_data_counter;
	reg                        fifo_wren_reg;

	//Reset signals
	wire                       reset;
	reg  [11:0]                reset_counter;
	reg                        reset_signal_reg;
	
	//LED Blinky Transfer registers
	wire [35:0]                LED;
	reg                        led_start_transfer;
	reg                        led_start_transfer_reg;
	wire                       led_transfer_in_received;
	reg   [7:0]                led_host_transfer_byte;
	wire  [7:0]                led_device_transfer_byte;

	//LED BLinky Signals
	reg   [23:0]               timer_value;	
	reg   [7:0]                seed_value;
	reg   [23:0]               led_delay_counter;
	reg   [3:0]                random_number;
	reg   [3:0]                random_num_counter;
	wire                       random_number_generated;
	reg                        start_blinky;
	reg   [3:0]                static_load_value;	
	reg   [3:0]                shift_count_value;
	
	//Image buffers
	reg   [215:0]              led_reg;
	wire  [35:0]               led_array;
	reg   [215:0]              active_image_array;
	reg                        idle_leds;
	reg                        load_leds;
	wire                       new_image_ready;
	wire                       exit_blinky;
	wire                       load_ept_image;
	wire                       load_face_1_image;
	wire                       load_tri_wave_image;
	
	//Image Files
	wire  [215:0]              ept_logo_image_array; 
	reg                        img_ept_logo_en;
	wire  [215:0]              face_1_image_array;
	reg                        img_face_1_en;
	wire  [215:0]              block_image_array;
	reg                        img_block_en;
	wire  [215:0]              tri_wave_image_array; 
	reg                        img_tri_wave_en;
	
	//Block Transfer for Image
	reg                        block_out_image_start;
	wire                       block_in_image_rcv;
	wire                       block_byte_image_ready;
	wire                       block_image_busy;
	wire  [7:0]                ept_length_image;
	wire  [7:0]                uc_length_image;
	reg   [7:0]                block_out_byte_image;
	wire  [7:0]                block_in_data_image;
	reg   [7:0]                block_in_image_counter;
	reg   [7:0]                block_in_image_buffer [31:0];	
	reg   [2:0]                state_block_image;
	wire                       load_block_image;

    //Block Slide Switch
	reg                        block_slide_switch_start;
	wire                       block_slide_switch_rcv;
	wire                       block_byte_slide_switch_ready;
	wire                       block_slide_switch_busy;
	reg   [7:0]                ept_length_slide_switch;
    wire  [7:0]                ept_slide_switch_length;
	wire  [7:0]                block_out_byte_slide_switch;
	wire  [7:0]                block_in_data_slide_switch;
	reg                        block_slide_switch_go;
	reg                        block_slide_switch_reg;
	reg   [7:0]                block_slide_switch_counter;
    reg   [1:0]                block_slide_switch_state;

	
	
	//Block Memory Registers
    //reg   [7:0]                mem_array [256:0];
	//reg   [2:0]                mem_array_index;
	
	//Registers for Test
	wire  [3:0]                active_serial_state_out;
	wire  [11:0]               active_transfer_test_out;
	
	//Logic Analyzer Registers
	wire  [7:0]                XIO_6A;
	wire  [7:0]                XIO_6B;
	wire  [7:0]                XIO_6C;
	wire  [7:0]                XIO_6D;
	wire  [7:0]                XIO_6E;
	
	
	
	
	
	//Test Signals


//***************************************************************************
//* 	Signal Assignments	
//***************************************************************************
   
   //Clock and Reset
   assign            aa[1] = CLK_66MHZ;
   assign            aa[0] = RST_N;
   assign            RST_N = reset_signal_reg;
    
    //Control Register Map
   assign            transfer_in_loop_back = control_register[0];
   assign            load_block_image      = control_register[1];
   assign            load_ept_image        = control_register[2];
   //assign            load_face_1_image     = control_register[3];
   assign            load_face_1_image     = 1'b0;
  assign            load_tri_wave_image     = control_register[3];
   
   //Block to host byte
   //assign            block_out_byte = mem_array[block_out_counter];
   assign            uc_length = 8'h8;
   
   //Slide Switch Block
   assign            block_out_byte_slide_switch = {6'h0,SLIDE_SWITCH_2,SLIDE_SWITCH_1};
   
   //New image has been transferred into memory
   assign            new_image_ready = 1'b0;
   
   //Terminate the Blinky code
   assign            exit_blinky = 1'b0;
   
   //Inputs from the XIO_6 Bus
   assign            XIO_6A = {XIO_7[3],XIO_6[31:28]};
   assign            XIO_6B = {XIO_6[27:20]};
   assign            XIO_6C = {XIO_6[19:12]};
   assign            XIO_6D = {XIO_6[11:4]};
   assign            XIO_6E = {XIO_6[3:0],4'h00};
   
   //Feedback signals from FPGA
/*   assign           XIO_1[0] = block_in_data[0]; 
   assign           XIO_1[1] = block_in_data[1]; 
   assign           XIO_1[2] = block_in_data[2]; 
   assign           XIO_1[3] = block_in_data[3]; 
   assign           XIO_1[4] = block_in_data[4];
   assign           XIO_1[5] = block_in_data[5];
   assign           XIO_1[6] = block_in_data[6];
   assign           XIO_1[7] = block_in_data[7];

   assign           XIO_3[0] = control_register[1]; 
   assign           XIO_3[1] = control_register[2]; 
   assign           XIO_3[2] = control_register[3]; 
   assign           XIO_3[3] = control_register[4]; 
   assign           XIO_3[4] = control_register[5]; 
   assign           XIO_3[5] = control_register[6]; 
   
   assign           XIO_5[0] = control_register[7]; 
   assign           XIO_5[1] = block_in_rcv; 
   assign           XIO_5[2] = 0; 
   assign           XIO_5[3] = 0; 
   assign           XIO_5[4] = 0; 
   assign           XIO_5[5] = 0; 
*/
  // Top Row of LEDs   
  assign            XIO_6[0] = start_blinky ? LED[0] : 1'bz; //UB22
  assign            XIO_6[1] = start_blinky ? LED[1] : 1'bz; //UB23
  assign            XIO_6[2] = start_blinky ? LED[2] : 1'bz; //UB24
  assign            XIO_6[3] = start_blinky ? LED[3] : 1'bz; //UB25
  assign            XIO_6[4] = start_blinky ? LED[4] : 1'bz; //UB26
  assign            XIO_6[5] = start_blinky ? LED[5] : 1'bz; //UB27
  
  // Second from Top Row of LEDs   
  assign            XIO_6[6] = start_blinky ? LED[6] : 1'bz; //UB28
  assign            XIO_6[7] = start_blinky ? LED[7] : 1'bz; //UB29
  assign            XIO_6[8] = start_blinky ? LED[8] : 1'bz; //UB30
  assign            XIO_6[9] = start_blinky ? LED[9] : 1'bz; //UB31
  assign            XIO_6[10] = start_blinky ? LED[10] : 1'bz; //UB32 
  assign            XIO_6[11] = start_blinky ? LED[11] : 1'bz; //UB33

  // Third from Top Row of LEDs   
  assign            XIO_6[12] = start_blinky ? LED[12] : 1'bz; //UB34
  assign            XIO_6[13] = start_blinky ? LED[13] : 1'bz; //UB35
  assign            XIO_6[14] = start_blinky ? LED[14] : 1'bz; //UB36 Cannot use pin 91 as output
  assign            XIO_6[15] = start_blinky ? LED[15] : 1'bz; //UB37 Cannot use pin 90 as output
  assign            XIO_6[16] = start_blinky ? LED[16] : 1'bz; //UB38 Cannot use pin 89 as output
  assign            XIO_6[17] = start_blinky ? LED[17] : 1'bz; //UB39

  // Fourth from Top Row of LEDs   
  assign            XIO_6[18] = start_blinky ? LED[18] : 1'bz; //UB40
  assign            XIO_6[19] = start_blinky ? LED[19] : 1'bz; //UB41
  assign            XIO_6[20] = start_blinky ? LED[20] : 1'bz; //UB42
  assign            XIO_6[21] = start_blinky ? LED[21] : 1'bz; //UB43
  assign            XIO_6[22] = start_blinky ? LED[22] : 1'bz; //UB44
  assign            XIO_6[23] = start_blinky ? LED[23] : 1'bz; //UB45

  // Fifth from Top Row of LEDs   
  assign            XIO_6[24] = start_blinky ? LED[24] : 1'bz; //UB46
  assign            XIO_6[25] = start_blinky ? LED[25] : 1'bz; //UB47
  assign            XIO_6[26] = start_blinky ? LED[26] : 1'bz; //UB48
  assign            XIO_6[27] = start_blinky ? LED[27] : 1'bz; //UB49
  assign            XIO_6[28] = start_blinky ? LED[28] : 1'bz; //UB50
  assign            XIO_6[29] = start_blinky ? LED[29] : 1'bz; //UB51

  // Sixth from Top Row of LEDs   
  assign            XIO_6[30] = start_blinky ? LED[30] : 1'bz; //UB52
  assign            XIO_6[31] = start_blinky ? LED[31] : 1'bz; //UB53
  assign            XIO_7[0] = start_blinky ? LED[32] : 1'bz; //UB68
  assign            XIO_7[1] = start_blinky ? LED[33] : 1'bz; //UB69
  assign            XIO_7[2] = start_blinky ? LED[34] : 1'bz; //UB70
  assign            XIO_7[3] = start_blinky ? LED[35] : 1'bz; //UB71
  
  
//**************************************************
//* 	Reset Signal	
//**************************************************
	always @(posedge CLK_66MHZ or negedge RST)
	begin
	  if(!RST)
	  begin
			reset_signal_reg <= 1'b0;
			reset_counter <= 0;
	  end
	  else
	  begin
		if( reset_counter < GLOBAL_RESET_COUNT )
		begin
			reset_signal_reg <= 1'b0;
			reset_counter <= reset_counter + 1'b1;
		end
		else
		begin
			reset_signal_reg <= 1'b1;
		end
	  end
	end

   //-----------------------------------------------
   // Register the incoming UART signal
   // 
   //-----------------------------------------------

  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
     if(!RST_N)
	 begin
	     uart_meta <= 1'b0;
		 uart_mid <= 1'b0;
		 uart_txd <= 1'b0;
	end
	else
	 begin
	     uart_meta <= UART_IN;
		 uart_mid <= uart_meta;
		 uart_txd <= uart_mid;
	 end
  
  end


   //-----------------------------------------------
   // Trigger Latch latches the trigger byte coming
   // from the host.
   //-----------------------------------------------

  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
     if(!RST_N)
	     trigger_from_host_latch <= 0;
	else
	 begin
	     if(trigger_in_byte)
		    trigger_from_host_latch <= trigger_in_byte;
	 end
  
  end

   //-----------------------------------------------
   // Trigger Out Byte. Device can 
   // send the trigger byte to the host.
   //-----------------------------------------------

  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
     if(!RST_N)
	     trigger_out <= 0;
	else
	 begin
         // Push Button swtiches, a button press will send 
         // send the trigger byte to the host.
	     if(!PB_SWITCH_1)
		    trigger_out <= PB_SWITCH_1_ASSERT;
	     else if(!PB_SWITCH_2)
		    trigger_out <= PB_SWITCH_2_ASSERT;
	     else
		    trigger_out <= 0;
	 end
  
  end

  
   //-----------------------------------------------
   // Transfer to Host in Loop Back Mode 
   //-----------------------------------------------
    always @(posedge CLK_66MHZ or negedge RST_N) 
	begin
	     if (!RST_N)
		 begin
			 transfer_out_reg <= 1'b0;
			 transfer_to_host_counter <= 0;
			 transfer_to_host_state <= TRANSFER_HOST_IDLE;
	     end 
		 else 
		 begin
			     transfer_out_reg <= 1'b0;
			case(transfer_to_host_state)
             TRANSFER_HOST_IDLE:
			     if(transfer_in_received & transfer_in_loop_back)	
                 begin			 
			         transfer_out_byte = transfer_in_byte;
			         transfer_to_host_state <= TRANSFER_HOST_LOOPBACK_START;
			     end
             TRANSFER_HOST_LOOPBACK_START:
		         if(transfer_to_host_counter < 4'hf)
			     begin
		             transfer_to_host_counter <= transfer_to_host_counter + 1'd1;
			         transfer_to_host_state <= TRANSFER_HOST_LOOPBACK_START;
			     end
				 else
				 begin
			         transfer_out_reg <= 1'b1;
			         transfer_to_host_state <= TRANSFER_HOST_LOOPBACK_COMPLETE;				 
				 end
             TRANSFER_HOST_LOOPBACK_COMPLETE:
			     begin
			         transfer_out_reg <= 1'b0;
					 transfer_to_host_counter <= 0;
			         transfer_to_host_state <= TRANSFER_HOST_IDLE;
				 end
			 default:
			      begin
			          transfer_out_reg <= 1'b0;
			          transfer_to_host_counter <= 0;
			          transfer_to_host_state <= TRANSFER_HOST_IDLE;
				  end
			 endcase
	     end
    end

   //-----------------------------------------------
   // Block In write to memory array is driven by block_byte_ready
   //-----------------------------------------------
/*  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
    if(!RST_N)
	begin
		 fifo_wren_reg <= 1'b0;
		 block_in_data_counter <= 0;
	end
	else 
	begin
	     if(block_in_rcv & block_byte_ready & !fifo_wren_reg)
		 begin
		     fifo_wren_reg <= 1'b1;
		 end
	     else if(block_in_rcv & block_byte_ready & fifo_wren_reg)
		 begin
		     fifo_wren_reg <= 1'b1;
			 mem_array[block_in_data_counter] <= block_in_data;
		 end
	     else if(block_in_rcv & !block_byte_ready & fifo_wren_reg)
		 begin
		     fifo_wren_reg <= 1'b0;
			 block_in_data_counter <= block_in_data_counter + 1'd1;
		 end
	     else if(!block_in_rcv)
		 begin
		     fifo_wren_reg <= 1'b0;
		     block_in_data_counter <= 0;
		 end
	end 
  end
*/   

   //-----------------------------------------------
   // Detect Slide Switch Block Transfer Start
   //-----------------------------------------------
  always @(trigger_in_byte or RST_N)
  begin
     if(!RST_N)
	     block_slide_switch_go = 1'b0;
	 else if((trigger_in_byte[2]))
	     block_slide_switch_go = 1'b1;
     else
 	     block_slide_switch_go = 1'b0;
 
  end
  
   //-----------------------------------------------
   // Registers to start Block Transfer Out
   /*	reg                        block_slide_switch_go;
	reg                       block_slide_switch_reg;
	reg  [7:0]                block_slide_switch_counter;
	wire                       block_slide_switch_busy;
	reg  [7:0]                block_slide_switch_state;
	wire  [7:0]                ept_length_slide_switch;
	reg   [7:0]                block_out_byte_slide_switch;
	wire  [7:0]                block_in_data_slide_switch;
	BLK_SLIDE_SWITCH_IDLE
	BLK_SLIDE_SWITCH_COUNT
    */
   //-----------------------------------------------
  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
    if(!RST_N)
	begin
		 block_slide_switch_start <= 1'b0;
		 block_slide_switch_reg <= 1'b0;
		 block_slide_switch_counter <= 0;
		 block_slide_switch_state <= BLK_SLIDE_SWITCH_IDLE;
		 //ept_length_slide_switch <= BLOCK_SLIDE_SWITCH_LENGTH;
	end
	else
	begin
	     case(block_slide_switch_state)
		     BLK_SLIDE_SWITCH_IDLE:
	             if(block_slide_switch_go)
				 begin
		             block_slide_switch_reg <= 1'b1;
		             block_slide_switch_state <= BLK_SLIDE_SWITCH_COUNT;
				 end
				 else
				    block_slide_switch_reg <= 1'b0;
			 BLK_SLIDE_SWITCH_COUNT:
		         begin
			          block_slide_switch_start <= 1'b1;
	                 if(block_byte_slide_switch_ready)
		             begin
		                 block_slide_switch_counter <= block_slide_switch_counter + 1'd1;
		             end
		             else if(block_slide_switch_counter >= BLOCK_SLIDE_SWITCH_LENGTH )
		             begin
		                 block_slide_switch_counter <= 0;
			             block_slide_switch_start <= 1'b0;
		                 block_slide_switch_state <= BLK_SLIDE_SWITCH_IDLE;
		             end
				 end
			 default:
			      begin
		                 block_slide_switch_counter <= 0;
			             block_slide_switch_start <= 1'b0;
		                 block_slide_switch_state <= BLK_SLIDE_SWITCH_IDLE;
				  end
			 endcase
	end
  end

   //-----------------------------------------------
   // Block Out
   //-----------------------------------------------
/*  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
    if(!RST_N)
	begin
		state_s_switch_data <= BLOCK_LOOPBACK_IDLE;
	end
	else 
	begin
	     if(block_in_loopback) 
		 begin
		     case(state_block_loopback)
			 BLOCK_LOOPBACK_IDLE:
			 begin
			     if(block_out_reg)
				    state_block_loopback <= BLOCK_LOOPBACK_INIT;
			 end
			 BLOCK_LOOPBACK_INIT:
			 begin
				 state_block_loopback <= FIFO_EN_HIGH;
		     end
			 FIFO_EN_HIGH:
		     begin
	             if(block_byte_ready )
				 begin
				     state_block_loopback <= FIFO_EN_LOW;
				 end
				 else
				 begin
				     state_block_loopback <= FIFO_EN_HIGH;
				 end
		     end
			 FIFO_EN_LOW:
		     begin
				 if(block_out_counter == 0)
				 begin
					 if(block_out_reg)
				         state_block_loopback <= BLOCK_LOOPBACK_IDLE;
				 end
	             else if(!block_byte_ready )
				 begin
				     state_block_loopback <= FIFO_EN_HIGH;
				 end
			 end
			 endcase
		 end
		 else
		 begin
		     state_block_loopback <= BLOCK_LOOPBACK_IDLE;
		 end
	end 
  end
*/

   //-----------------------------------------------
   // Detect Block Transfer Out start
   //-----------------------------------------------
  always @(block_out_reg or trigger_in_byte or RST_N)
  begin
     if(!RST_N)
	     start_block_transfer = 1'b0;
	 else if(block_out_reg)
	     start_block_transfer = 1'b0;
	 else if((trigger_in_byte[7]))
	     start_block_transfer = 1'b1;
  
  end
  
   //-----------------------------------------------
   // Registers to start Block Transfer Out
   //-----------------------------------------------
  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
    if(!RST_N)
	begin
		 block_out_reg <= 1'b0;
		 start_block_transfer_reg <= 1'b0;
		 block_out_counter <= 0;
		 block_transfer_out_state <= BLK_TRANSFER_OUT_IDLE;
	end
	else
	begin
	     case(block_transfer_out_state)
		     BLK_TRANSFER_OUT_IDLE:
	             if(start_block_transfer )
				 begin
		             start_block_transfer_reg <= 1'b1;
		             block_transfer_out_state <= BLK_TRANSFER_OUT_COUNT;
				 end
			 BLK_TRANSFER_OUT_COUNT:
		         begin
			          block_out_reg <= 1'b1;
	                 if(block_byte_ready)
		             begin
		                 block_out_counter <= block_out_counter + 1'd1;
		             end
		             else if(block_out_counter >= DUE_GPIO_MAX_NUMBER )
		             begin
		                 block_out_counter <= 0;
			             block_out_reg <= 1'b0;
		                 block_transfer_out_state <= BLK_TRANSFER_OUT_IDLE;
		             end
				 end
			 default:
			      begin
		                 block_out_counter <= 0;
			             block_out_reg <= 1'b0;
		                 block_transfer_out_state <= BLK_TRANSFER_OUT_IDLE;
				  end
			 endcase
	end
  end

   //-----------------------------------------------
   // Block In read fifo enable is driven by block_byte_ready
   //-----------------------------------------------
  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
    if(!RST_N)
	begin
		//state_block_loopback <= BLOCK_LOOPBACK_IDLE;
	end
	else 
	begin
		     case(block_out_counter)
			 0:
			 begin
			     block_out_byte <= XIO_1;
			 end
			 1:
			 begin
				 block_out_byte <= {1'b1,XIO_3,1'b0};
		     end
			 2:
		     begin
	             block_out_byte <= {1'b1,XIO_4,1'b0};
		     end
			 3:
		     begin
				 block_out_byte <= XIO_6A;
			 end
			 4:
			 begin
			     block_out_byte <= XIO_6B;
			 end
			 5:
			 begin
			     block_out_byte <= XIO_6C;
			 end
			 6:
		     begin
				 block_out_byte <= XIO_6D;
			 end
			 7:
			 begin
			     block_out_byte <= XIO_6E;
			 end
			 endcase
	end
  end


   //-----------------------------------------------
   // Load Image into local buffer
   //-----------------------------------------------

  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
    if(!RST_N)
	begin
	     img_block_en <= 1'b0;
	     img_ept_logo_en <= 1'b0;
		 img_face_1_en <= 1'b0;
	     img_tri_wave_en <= 1'b0;
		 active_image_array <= 0;
    end
	else 
	begin
	   if(load_block_image)
	   begin
	      active_image_array <= block_image_array;
          img_block_en <= 1'b1;
	   end
	   else
	   begin
	      img_block_en <= 1'b0;
	   end

       if(load_ept_image)
	   begin
	      active_image_array <= ept_logo_image_array;
          img_ept_logo_en <= 1'b1;
	   end
	   else
	   begin
	      img_ept_logo_en <= 1'b0;
	   end

       if(load_face_1_image)
	   begin
	      active_image_array <= face_1_image_array;
          img_face_1_en <= 1'b1;
	   end
	   else
	   begin
	      img_face_1_en <= 1'b0;
	   end
	   
       if(load_tri_wave_image)
	   begin
	      active_image_array <= tri_wave_image_array;
          img_tri_wave_en <= 1'b1;
	   end
	   else
	   begin
	      img_tri_wave_en <= 1'b0;
	   end


	   end 
  end

   //-----------------------------------------------
   // Block In write to Image buffer driven by block_byte_ready_image
   //-----------------------------------------------
  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
    if(!RST_N)
	begin
		 state_block_image <= BLOCK_IMAGE_IDLE;
		 block_in_image_counter <= 0;
	end
	else 
	begin
       case(state_block_image)
       BLOCK_IMAGE_IDLE:
       begin
	     if(block_in_image_rcv)
		 begin
			 state_block_image <= BLOCK_IMAGE_WAIT_FOR_READY;
		 end
       end
       BLOCK_IMAGE_WAIT_FOR_READY:
       begin
	     if(block_in_image_rcv & block_byte_image_ready)
		 begin
			 state_block_image <= BLOCK_IMAGE_STORE_BYTE;
		 end
	     else if(!block_in_image_rcv)
		 begin
		     block_in_image_counter <= 0;
			 state_block_image <= BLOCK_IMAGE_IDLE;
		 end
       end
	   BLOCK_IMAGE_STORE_BYTE:
       begin
	     if(block_in_image_rcv & !block_byte_image_ready)
		 begin
			 block_in_image_buffer[block_in_image_counter] <= block_in_data_image;
			 state_block_image <= BLOCK_IMAGE_CHECK_COMPLETE;
		 end
	     else if(!block_in_image_rcv)
		 begin
		     block_in_image_counter <= 0;
			 state_block_image <= BLOCK_IMAGE_IDLE;
		 end
       end
       BLOCK_IMAGE_CHECK_COMPLETE:
       begin
		 block_in_image_counter <= block_in_image_counter + 1'd1;

		 if(block_in_image_rcv & block_byte_image_ready)
		 begin
			 state_block_image <= BLOCK_IMAGE_STORE_BYTE;
		 end
	     else if(block_in_image_rcv)
		 begin
		     //block_in_image_counter <= 0;
			 state_block_image <= BLOCK_IMAGE_WAIT_FOR_READY;
		 end
	     else if(!block_in_image_rcv)
		 begin
		     block_in_image_counter <= 0;
			 state_block_image <= BLOCK_IMAGE_IDLE;
		 end
		end
       endcase
	end 
  end
   
   //-----------------------------------------------
   // Load Timer Value
   //-----------------------------------------------

  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
    if(!RST_N)
	     timer_value <= TIMER_LOW_LIMIT;
	else 
	begin
	     if(trigger_in_byte[4])
	         timer_value[15:8] <= control_register_in_byte;
	     else if(trigger_in_byte[5])
	         timer_value[23:16] <= control_register_in_byte;
	end 
  end

   //-----------------------------------------------
   // LED Blinky start 
   //-----------------------------------------------
  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
    if(!RST_N)
         start_blinky <= 1'b0;
	else 
	begin
	    if(control_register[7:4] > 0)
            start_blinky <= 1'b1;
		else
            start_blinky <= 1'b0;		
 	end 
  end
 
   //-----------------------------------------------
   // LED Delay Timer Counter 
   //-----------------------------------------------
  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
    if(!RST_N)
	     led_delay_counter <= TIMER_LOW_LIMIT;
	else 
	begin
	     if(state[SELECT_MODE])
		    led_delay_counter <= HIGH_Z_TIMER_VALUE;
		 else if(state[WAIT_FOR_TIMER_HIGH_Z])
	        led_delay_counter <= led_delay_counter - 1'd1;
		 else if(state[LOAD_LED_ARRAY])
	        led_delay_counter <= timer_value;
		 else if(state[WAIT_FOR_TIMER_ACTIVE])
	        led_delay_counter <= led_delay_counter - 1'd1;
	end 
  end

   //-----------------------------------------------
   // Random Number array
   //-----------------------------------------------
  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
     if(!RST_N)
	 begin
	     random_number <= 6'h0;
		 random_num_counter <= 0;
	 end
	else 
	begin
	    if(random_num_counter < 6'h4 )
	    begin
	        random_num_counter <= random_num_counter + 1'd1;
			random_number[random_num_counter] <= random_number_generated;
		end
	    else 
		begin
	        random_num_counter <= 0;
			random_number[random_num_counter] <= random_number_generated;
		end
	end
  end

   //-----------------------------------------------
   // Select the LED Function from the control
   // register bits 7 to 4 
   // The led_reg has five rows of 20 columns
   //-----------------------------------------------
  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
    if(!RST_N)
	     led_reg <= 0;
	else 
	begin
	     if(state[LOAD_IMAGE])
	         led_reg <= active_image_array;
	
        if (state[SELECT_MODE])
	    begin
	         case(control_register[7:4])
		       RANDOM_NUMBER:
		          led_reg <= random_number;
		       SHIFT_IMAGE_RIGHT:
			   begin
		          led_reg[23:1] <= led_reg[22:0];    // Top Row of LEDs
				  led_reg[0] <= led_reg[23];
				  
		          led_reg[47:25] <= led_reg[46:24];  // Second from Top Row of LEDs 
				  led_reg[24] <= led_reg[47];

		          led_reg[71:49] <= led_reg[70:48];  // Third from Top Row of LEDs
				  led_reg[48] <= led_reg[71];

		          led_reg[95:73] <= led_reg[94:72];  // Fourth from Top Row of LEDs
				  led_reg[72] <= led_reg[95];

		          led_reg[119:97] <= led_reg[118:96];  // Fifth from Top Row of LEDs
				  led_reg[96] <= led_reg[119];

		          led_reg[143:121] <= led_reg[142:120];  // Sixth from Top Row of LEDs
				  led_reg[120] <= led_reg[143];

		          led_reg[167:145] <= led_reg[166:144];  // Sevenh from Top Row of LEDs
				  led_reg[144] <= led_reg[167];

		          led_reg[191:169] <= led_reg[190:168];  // Eighth from Top Row of LEDs
				  led_reg[168] <= led_reg[191];

		          led_reg[215:193] <= led_reg[214:192];  // Ninth from Top Row of LEDs
				  led_reg[192] <= led_reg[215];

			   end
		       SHIFT_IMAGE_LEFT:
			   begin
		          led_reg[22:0] <= led_reg[23:1];    // Top Row of LEDs
				  led_reg[23] <= led_reg[0];
				  
		          led_reg[46:24] <= led_reg[47:25];  // Second from Top Row of LEDs 
				  led_reg[47] <= led_reg[24];

		          led_reg[70:48] <= led_reg[71:49];  // Third from Top Row of LEDs
				  led_reg[71] <= led_reg[48];

		          led_reg[94:72] <= led_reg[95:73];  // Fourth from Top Row of LEDs
				  led_reg[95] <= led_reg[72];

		          led_reg[118:96] <= led_reg[119:97];  // Fifth from Top Row of LEDs
				  led_reg[119] <= led_reg[96];

		          led_reg[142:120] <= led_reg[143:121];  // Sixth from Top Row of LEDs
				  led_reg[143] <= led_reg[120];

		          led_reg[166:144] <= led_reg[167:145];  // Seventh from Top Row of LEDs
				  led_reg[167] <= led_reg[144];

		          led_reg[190:168] <= led_reg[191:169];  // Eighth from Top Row of LEDs
				  led_reg[191] <= led_reg[168];

		          led_reg[214:192] <= led_reg[215:121];  // Ninth from Top Row of LEDs
				  led_reg[215] <= led_reg[192];

			   end
		       SCROLL_UP:
			   begin
		          led_reg[23:0] <= led_reg[47:24];    // Top Row of LEDs

		          led_reg[47:24] <= led_reg[71:48];  // Second from Top Row of LEDs 
				  				  
		          led_reg[71:48] <= led_reg[95:72];  // Third from Top Row of LEDs

		          led_reg[95:72] <= led_reg[119:96];  // Fourth from Top Row of LEDs

		          led_reg[119:96] <= led_reg[143:120];  // Fifth from Top Row of LEDs

		          led_reg[143:120] <= led_reg[167:144];  // Sixth from Top Row of LEDs

		          led_reg[167:144] <= led_reg[191:168];  // Seventh from Top Row of LEDs

		          led_reg[191:168] <= led_reg[215:192];  // Eighth from Top Row of LEDs

		          led_reg[215:192] <= led_reg[23:0];  // Ninth from Top Row of LEDs
				  end
		       SCROLL_DOWN:
			   begin
		          led_reg[23:0] <= led_reg[215:192];    // Top Row of LEDs

		          led_reg[47:24] <= led_reg[23:0];  // Second from Top Row of LEDs 
				  				  
		          led_reg[71:48] <= led_reg[47:24];  // Third from Top Row of LEDs

		          led_reg[95:72] <= led_reg[71:48];  // Fourth from Top Row of LEDs

		          led_reg[119:96] <= led_reg[95:72];  // Fifth from Top Row of LEDs

		          led_reg[143:120] <= led_reg[119:96];  // Sixth from Top Row of LEDs

		          led_reg[167:144] <= led_reg[143:120];  // Seventh from Top Row of LEDs

		          led_reg[191:168] <= led_reg[167:144];  // Eighth from Top Row of LEDs

		          led_reg[215:192] <= led_reg[191:168];  // Ninth from Top Row of LEDs
				  end
		       STATIC_VALUE:
		          led_reg <= active_image_array;
		       STATIC_IMAGE:
		          led_reg <= active_image_array;
		       BLINK_IMAGE:
		          led_reg <= active_image_array;
		       SHIFT_RIGHT:
		          led_reg <= active_image_array;
		       SHIFT_LEFT:
		          led_reg <= active_image_array;
		       SHIFT_IMAGE_DOWN:
		          led_reg <= active_image_array;
		       SHIFT_IMAGE_UP:
		          led_reg <= active_image_array;
		       default:
		          led_reg <= 143'h0;
		      endcase 
        end	
    end
  end
  

   //-----------------------------------------------
   // Set the LED outputs 
   //-----------------------------------------------
  always @(posedge CLK_66MHZ or negedge RST_N)
  begin
    if(!RST_N)
	begin
	     idle_leds <= 1'b0;
		 load_leds <= 1'b0;
	end
	else 
	begin
	   if(state[WAIT_FOR_TIMER_HIGH_Z])
	      idle_leds <= 1'b1;
	   else
	      idle_leds <= 1'b0; 
	
	   if(state[LOAD_LED_ARRAY])
	      load_leds <= 1'b1;
	   else
	      load_leds <= 1'b0; 
	end
  end

     //-----------------------------------------------
   // State Machine: Send Pattern to LEDs 
   //-----------------------------------------------
   // Next State Logic
   always @(posedge CLK_66MHZ or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     state <= 8'h00;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

     // State Definitions
   always @ ( state or start_blinky or random_number or
		      led_reg or led_delay_counter or control_register or
			  timer_value or shift_count_value) 
     begin
	     next = 8'h00;

	if (state[IDLE])
	  begin
	     if (start_blinky)
	       next[LOAD_IMAGE] = 1'b1;
	     else
	       next[IDLE] = 1'b1;
	  end
	
	if (state[LOAD_IMAGE])
	  begin
	     if (start_blinky)
	       next[SELECT_MODE] = 1'b1;
	     else
	       next[LOAD_IMAGE] = 1'b1;
	  end
	
	if (state[SELECT_MODE])
	begin
       if(control_register[7:4] == 4'h0)        
            next[IDLE] = 1'b1;
         else		 
           next[UPDATE_IMAGE] = 1'b1;
	end

	if (state[UPDATE_IMAGE])
	begin
        next[WAIT_FOR_TIMER_HIGH_Z] = 1'b1;
    end
		
	if (state[WAIT_FOR_TIMER_HIGH_Z])
	begin
	    if(led_delay_counter == 0)
           next[LOAD_LED_ARRAY] = 1'b1;
		else
		  next[WAIT_FOR_TIMER_HIGH_Z] = 1'b1; 
    end
	
	if (state[LOAD_LED_ARRAY])
		if (led_delay_counter <= timer_value)
			next[LOAD_LED_ARRAY] = 1'b1;
		else 
			next[WAIT_FOR_TIMER_ACTIVE] = 1'b1;

	if (state[WAIT_FOR_TIMER_ACTIVE])
	begin
	    if(led_delay_counter == 0)
           next[CHECK_FOR_NEW_IMAGE] = 1'b1;
		else
           next[WAIT_FOR_TIMER_ACTIVE] = 1'b1;
    end

	if (state[CHECK_FOR_NEW_IMAGE])
	begin
		if (new_image_ready)
			next[LOAD_IMAGE] = 1'b1;
		else if(exit_blinky)
			next[IDLE] = 1'b1;
		else 
			next[SELECT_MODE] = 1'b1;
    end

			
`ifdef SIM
 	   if ( state == ( 1 << IDLE ))
		   state_name = "IDLE";
	   else if ( state == ( 1 << LOAD_IMAGE ))
		   state_name = "LOAD_IMAGE";
	   else if ( state == ( 1 << SELECT_MODE ))
		   state_name = "SELECT_MODE";
	   else if ( state == ( 1 << UPDATE_IMAGE ))
		   state_name = "UPDATE_IMAGE";
	   else if ( state == ( 1 << WAIT_FOR_TIMER_HIGH_Z ))
		   state_name = "WAIT_FOR_TIMER_HIGH_Z";
	   else if ( state == ( 1 << LOAD_LED_ARRAY ))
		   state_name = "LOAD_LED_ARRAY";
	   else if ( state == ( 1 << WAIT_FOR_TIMER_ACTIVE ))
		   state_name = "WAIT_FOR_TIMER_ACTIVE";
	   else if ( state == ( 1 << CHECK_FOR_NEW_IMAGE ))
		   state_name = "CHECK_FOR_NEW_IMAGE";
`endif


	end//end of state machine

   //-----------------------------------------------
   // Image Files
   //-----------------------------------------------
  image_block_in               IMG_BLOCK_IN_INST
  (
   .IMAGE_ARRAY                (block_image_array),
   .INPUT_BUFFER               (block_in_data_image),
   .INPUT_BUFFER_INDEX         (block_in_image_counter),
   .ENABLE                     (block_in_image_rcv)

   );	
   
  image_ept_logo               IMG_EPT_LOGO_INST
  (
   .IMAGE_ARRAY                (ept_logo_image_array),
   .ENABLE                     (img_ept_logo_en)

   );	
   
  image_face_1                 IMG_FACE_1_INST
  (
   .IMAGE_ARRAY                (face_1_image_array),
   .ENABLE                     (img_face_1_en)

   );	

  image_tri_wave               IMG_TRI_WAVE_INST
  (
   .IMAGE_ARRAY                (tri_wave_image_array),
   .ENABLE                     (img_tri_wave_en)

   );	
   
   
   //-----------------------------------------------
   // Update LED array
   //-----------------------------------------------
   led_update                  LED_UPDATE_INST
   (
	.CLK                        (CLK_66MHZ),
	.RST                        (RST_N),

	.LED_REG                    (led_reg[143:0]),
	.LED                        (LED),

	.LOAD_LEDS                  (load_leds),
	.IDLE_LEDS                  (idle_leds)


	);


   
   //-----------------------------------------------
   // Linear Feedback Shift Register
   //-----------------------------------------------
  lfsr                         LFSR_INST
  (
   .Q                          (random_number_generated),
   .CLK                        (CLK_66MHZ),
   .RST                        (RST_N),
   .SEED                       (led_device_transfer_byte[3:0]),
   .LOAD                       (trigger_in_byte[6])
   );	
	

   //-----------------------------------------------
   // State Machine: Control Register from Transfer In 
   //-----------------------------------------------
  /*
     Control Register Register Map
	 
	 control_register[0]  =  transfer_in_loop_back
	 control_register[1]  =  load_block_image//block_in_loopback 
	 control_register[2]  =  load_ept_image
	 control_register[3]  =  load_tri_wave_image//load_face_1_image
	 control_register[4]  =  LED Select Mode[0]
	 control_register[5]  =  LED Select Mode[1]
	 control_register[6]  =  LED Select Mode[2]
	 control_register[7]  =  LED Select Mode[3]
	 

  */
  active_control_register      ACTIVE_CONTROL_REG_INST
  (
   .CLK                        (CLK_66MHZ),
   .RST                        (RST_N),
   .TRANSFER_IN_RECEIVED       (control_register_in_received),
   .TRANSFER_IN_BYTE           (control_register_in_byte),

   .CONTROL_REGISTER           (control_register)
   );
   
   //-----------------------------------------------
   // Instantiate the EPT Library
   //-----------------------------------------------

	active_serial_library	   ACTIVE_SERIAL_LIBRARY_INST
	(	
	.aa                        (aa),
	.UART_OUT                  (UART_OUT),
	.UART_IN                   (uart_txd),

	.UC_IN                     (UC_IN),
	.UC_OUT                    (UC_OUT),
	
	.STATE_OUT                 (active_serial_state_out),
	.TEST_OUT                  (active_transfer_test_out) 
	
	);

   //-----------------------------------------------
   // Instantiate the EPT Library
   //-----------------------------------------------
wire [22*6-1:0]  uc_out_m;
eptWireOR # (.N(6)) wireOR (UC_OUT, uc_out_m);

/*
    Active Trigger Register Mapping
	
	trigger_in_byte[0]  =  Load Static Value
	trigger_in_byte[1]  =  Load shift_count_value
	trigger_in_byte[2]  =  Command Slide Switch Block to Transmit to Host
	trigger_in_byte[3]  =  
	trigger_in_byte[4]  =  Load Timer Value, timer_value[15:8]
	trigger_in_byte[5]  =  Load Timer Value, timer_value[23:16]
	trigger_in_byte[6]  =  Load linear_feedback_shift_register
	trigger_in_byte[7]  =  start_block_transfer = 1'b1
	

*/
	active_trigger             ACTIVE_TRIGGER_INST
	(
	 .uc_clk                   (CLK_66MHZ),
	 .uc_reset                 (RST_N),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 0*22 +: 22 ]),

	 .trigger_to_host          (trigger_out),
	 .trigger_to_device        (trigger_in_byte)
	
	);

    active_transfer            ACTIVE_TRANSFER_INST_1
	(
	 .uc_clk                   (CLK_66MHZ),
	 .uc_reset                 (RST_N),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 1*22 +: 22 ]),
	
	 .start_transfer           (transfer_out_reg),
	 .transfer_received        (transfer_in_received),
	 
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h1),

	 .transfer_to_host         (transfer_out_byte),
	 .transfer_to_device       (transfer_in_byte)	
    );
	
	active_transfer            ACTIVE_TRANSFER_INST_2
	(
	 .uc_clk                   (CLK_66MHZ),
	 .uc_reset                 (RST_N),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 2*22 +: 22 ]),
	
	 .start_transfer           (control_register_out_reg),
	 .transfer_received        (control_register_in_received),
	
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h2),

	 .transfer_to_host         (control_register_out_byte),
	 .transfer_to_device       (control_register_in_byte)	
	);
	
	
	active_block               BLOCK_TRANSFER_INST
	(
	 .uc_clk                   (CLK_66MHZ),
	 .uc_reset                 (RST_N),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 3*22 +: 22 ]),
	
	 .start_transfer           (block_out_reg),
	 .transfer_received        (block_in_rcv),
	 
	 .transfer_ready           (block_byte_ready),
	 .transfer_busy            (block_busy),

	 .ept_length               (ept_length),
	
	 .uc_addr                  (3'h4),
	 .uc_length                (DUE_GPIO_MAX_NUMBER),

	 .transfer_to_host         (block_out_byte),
	 .transfer_to_device       (block_in_data)
	
	);

	active_block               BLOCK_IMAGE_INST
	(
	 .uc_clk                   (CLK_66MHZ),
	 .uc_reset                 (RST_N),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 4*22 +: 22 ]),
	
	 .start_transfer           (block_out_image_start),
	 .transfer_received        (block_in_image_rcv),
	 
	 .transfer_ready           (block_byte_image_ready),
	 .transfer_busy            (block_image_busy),

	 .ept_length               (ept_length_image),
	
	 .uc_addr                  (3'h2),
	 .uc_length                (ept_length_image),

	 .transfer_to_host         (block_out_byte_image),
	 .transfer_to_device       (block_in_data_image)
	
	);

	active_block               BLOCK_SLIDE_SWITCH_INST
	(
	 .uc_clk                   (CLK_66MHZ),
	 .uc_reset                 (RST_N),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 5*22 +: 22 ]),
	
	 .start_transfer           (block_slide_switch_start),
	 .transfer_received        (block_slide_switch_rcv),
	 
	 .transfer_ready           (block_byte_slide_switch_ready),
	 .transfer_busy            (block_slide_switch_busy),

	 .ept_length               (ept_slide_switch_length),
	
	 .uc_addr                  (3'h5),
	 .uc_length                (BLOCK_SLIDE_SWITCH_LENGTH),

	 .transfer_to_host         (block_out_byte_slide_switch),
	 .transfer_to_device       (block_in_data_slide_switch)
	
	);

	
endmodule

