//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2018
//#
//#
//# File Name:  EPT_10M04_AF_S2_Top.v
//# Author:     R. Jolly
//# Date:       March 7, 2018
//# Revision:   A
//#
//# Development: USB to UART interface 
//# Application: Altera MAX10 FPGA
//# Description: This file contains verilog code which will allow access
//#              to an eight bit bus in the FPGA. The FPGA receives its commands via
//#				 USB.
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			03/7/18 	A			Created			RJJ
//#
//#									
//#
//#######################################################################
`timescale 1ns/1ps

`include "user_define.v"


//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_10M04_AF_S2_Top (

	
	input  wire                CLK_50MHZ,
	input  wire                CLK_32KHZ,
	input wire                 RST_N,
		
	output wire                CLK_50MHZ_ENABLE,
	output wire                CLK_32KHZ_ENABLE,
	
	//UART Signals
	input wire                 UART_IN,
	output wire                UART_OUT,
	
	//Unused signals
 	output reg  [7:0]          CONTROL_REGISTER,
	input  wire [7:0]          STATUS_REGISTER,
 	output wire [7:0]          RECEIVE_REGISTER,
 	input  wire [7:0]          SEND_REGISTER,

	output reg [7:0]         LED
	

	);

   //----------------------------------------------
   // Parameter Declarations                        
   //----------------------------------------------
   // UART Parameter Declarations                        
   parameter                   UART_IDLE                = 0,
                               RECEIVE_DATA				= 1,
                               STORE_BYTE		    	= 2,
                               SEND_TO_HOST			    = 3,
                               RECEIVE_COMPLETE			= 4,
                               WRITE_REGISTER 		    = 5,
                               LOAD_SHIFT_REG			= 6,
                               DELAY_WRITE_DATA_BYTE    = 7,
                               WRITE_DATA_BYTE			= 8,
                               WRITE_DATA_COMPLETE		= 9;
		 

`ifdef FAST_UART
		parameter COUNT_TO_DECIMAL       = 4;
		parameter COUNT_TO_DECIMAL_3     = 8'h36;
		parameter COUNT_TO_DECIMAL_2     = 8'h23; // 66 MHz Main Clock; 115200 UART Baud Rate
		parameter COUNT_TO_DECIMAL_1     = 8'h19; // 50 MHz Main Clock; 115200 UART Baud Rate  
`else
//		parameter COUNT_TO_DECIMAL       = 54;
		parameter COUNT_TO_DECIMAL_4     = 8'h6d;
		parameter COUNT_TO_DECIMAL_3     = 8'h36;
		parameter COUNT_TO_DECIMAL_2     = 8'h23; // 66 MHz Main Clock; 115200 UART Baud Rate
		parameter COUNT_TO_DECIMAL_1     = 8'h1b; // 50 MHz Main Clock; 115200 UART Baud Rate
		parameter COUNT_TO_DECIMAL       = 4;
`endif

`ifdef SIM
   	reg [8*30:1] state_name;
`endif
 	
	
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************
		
	//Finite State Machine control registers
   	reg [9:0]                  state, next;

	// UART Storage Registers
	reg [7:0]                  rcv_storage_reg;
	//reg [2:0]                  store_reg_state; 
	
	//Clock Registers
	wire                       clk16x;
	
	
	// UART Metastable Registers
	reg                        uart_in_reg;
	reg                        usb_wr_reg;
	
	//UART write_tx_byte_reg for signalling the UART transmitter
	//to start transmitting
	reg                        write_tx_byte_reg;

	//UART Interface signals
	wire [7:0]                  rx_data;
    wire                       rx_byte_present;
    wire                       UART_SERIAL_RX;
    wire                       en_16_x_baud;
    reg                        rx_byte_read;
	reg [7:0]                  tx_data_reg;
    wire                       tx_empty;
    reg                        write_tx_byte;
	
	
	//Transmit Registers
    reg                        transmit_data_ready;
    reg                        transmit_data_ready_reg;
	
	// CONTROL Register
	reg						control_update_reg;
	reg						status_update_reg;

	
//***************************************************************************
//* 	Signal Assignments	
//***************************************************************************
 
   //Date byte from UART 
	assign	RECEIVE_REGISTER = rcv_storage_reg;
 
//UART Clock control register  
`ifdef FAST_UART
	reg	[7:0]			counter_reg;
//	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_3 - 1) );
	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_1 - 1) );
`else
	reg	[7:0]			counter_reg;
//	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_3 - 1) );
	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_1 - 1) );
`endif

//=================================================================================
//	Receive Cycle: CONTROL_REGISTER, this register informs the ARM9 when sensor
//	data has been updated. uart_rdn
//=================================================================================


	always @(posedge CLK_50MHZ or negedge RST_N)
	begin
		if ( !RST_N )
		begin
			CONTROL_REGISTER <= 8'b0;
			control_update_reg <= 1'b0;
			status_update_reg <= 1'b0;
		end
		else
		begin
            if(state[SEND_TO_HOST] & !status_update_reg )
			begin
				CONTROL_REGISTER[3] <= 1'b1;
				status_update_reg <= 1'b1;
			end			
			else if( STATUS_REGISTER[3] & status_update_reg )
			begin
				CONTROL_REGISTER[3] <= 1'b0;
				status_update_reg <= 1'b1;
			end
			else if( !STATUS_REGISTER[3] & status_update_reg )
			begin
				status_update_reg <= 1'b0;
			end

			if ( state[RECEIVE_COMPLETE] & !control_update_reg )
			begin
				CONTROL_REGISTER[3] <= CONTROL_REGISTER[3] || 1'b1;
				control_update_reg <= 1'b1;
			end
			else if ( state[UART_IDLE] )
			begin
				control_update_reg <= 1'b0;
			end

			if (tx_empty)
			begin
				CONTROL_REGISTER[2] <= 1'b1;
			end
			else
			begin
				CONTROL_REGISTER[2] <= 1'b0;
			end
		end
	end

   //-----------------------------------------------
   // Transmit Cycle:
   //
   // UART Transfer
   // Start the Transmit by asserting the write_tx_byte Signal 
   //-----------------------------------------------
	always @(posedge CLK_50MHZ or negedge RST_N)
	begin
		if (!RST_N)
		begin
			write_tx_byte <= 1'b0;
			write_tx_byte_reg <= 1'b0;
		end
		else 
		begin
			if(state[DELAY_WRITE_DATA_BYTE] & !write_tx_byte_reg )
			begin
				write_tx_byte <= 1'b1;
				write_tx_byte_reg <= 1'b1;
			end
			else if( state[WRITE_DATA_COMPLETE] )
			begin
				write_tx_byte <= 1'b0;
				write_tx_byte_reg <= 1'b0;
		    end
		end	
	end // always @ (posedge CLK_50MHZ or negedge RST_N)   
   
   //-----------------------------------------------
   // UART Transfer
   // rx_byte_read register reads the available byte
   // in the UART receive module. 
   //-----------------------------------------------
   always @(posedge CLK_50MHZ or negedge RST_N)
     begin
	if (!RST_N)
	begin
	     rx_byte_read <= 1'b1;
		 rcv_storage_reg <= 8'he5;
	end
	else
	  begin
	     if (state[STORE_BYTE])
		 begin
	         rx_byte_read <= 1'b0;
			 rcv_storage_reg <= rx_data;
		 end
		 else
		 begin
	         rx_byte_read <= 1'b1;
			 rcv_storage_reg <= rcv_storage_reg;
		 end
	  end
     end // always @ (posedge CLK_50MHZ or negedge RST_N)

   //-----------------------------------------------
   // Transmit Cycle:
   // 
   // 
   // USB_WR_REG asserts and the UART state machine 
   // goes to state[LOAD_SHIFT_REG] 
   // and starts the transmit process.
   //-----------------------------------------------
   always @(posedge CLK_50MHZ or negedge RST_N)
     begin
	if (!RST_N)
		begin
	     usb_wr_reg <= 1'b0;
         transmit_data_ready <= 1'b0;
		 transmit_data_ready_reg <= 0;
		end
	else
	begin
	  if ( STATUS_REGISTER[1] & !transmit_data_ready_reg )
	  begin
         usb_wr_reg <= 1'b1;
         transmit_data_ready <= 1'b0;
         transmit_data_ready_reg <= 1'b1;
         tx_data_reg <= SEND_REGISTER;
	  end
	  else if ( STATUS_REGISTER[1] & transmit_data_ready_reg )
	  begin
         transmit_data_ready <= 1'b1;
         transmit_data_ready_reg <= 1'b1;
	  end
	  else if ( state[WRITE_DATA_BYTE] )
	  begin
         usb_wr_reg <= 1'b0;
         transmit_data_ready_reg <= 1'b0;
	  end
	  else if ( state[WRITE_DATA_COMPLETE] )
	  begin
         transmit_data_ready <= 1'b0;
         transmit_data_ready_reg <= 1'b0;
	  end
	  else if ( state[UART_IDLE] )
	  begin
         usb_wr_reg <= 1'b0;
         transmit_data_ready_reg <= 0;
	  end
	  end
    end // always @ (posedge CLK_50MHZ or negedge RST_N)



   //-----------------------------------------------
   // UART Transfer
   // Register the UART_IN input with uart_in_reg. 
   //-----------------------------------------------
   always @(posedge CLK_50MHZ or negedge RST_N)
     begin
	if (!RST_N)
	 uart_in_reg <= 1'b0;
	else
	  begin
	     if ( !UART_IN )
	       uart_in_reg <= 1'b0;//Hold the UART Receive in the off position
	     else 
	       uart_in_reg <= 1'b0;
	  end
     end // always @ (posedge CLK_50MHZ or negedge RST_N)

	  
   //--------------------------------------------------
   // UART Transfer
   // UART State Machine
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK_50MHZ or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     state <= 10'h00;
	     state[UART_IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

   always @ ( /*AUTOSENSE*/state or usb_wr_reg or tx_empty
		 or uart_in_reg or rx_byte_present or status_update_reg
		 or UART_OUT or transmit_data_ready)
     begin
	next = 10'h00;

	if (state[UART_IDLE])
	begin
        if (rx_byte_present)
			next[RECEIVE_DATA] = 1'b1;
		else if(usb_wr_reg)
			next[WRITE_REGISTER] = 1'b1;
		else
			next[UART_IDLE] = 1'b1;
	end

	if (state[RECEIVE_DATA])
	       next[STORE_BYTE] = 1'b1;

	  if(state[STORE_BYTE]) 
	  begin
	     if (!rx_byte_present)
	       next[SEND_TO_HOST] = 1'b1;
	     else
	         next[STORE_BYTE] = 1'b1;
	  end 	     

	if (state[SEND_TO_HOST]) 
	  begin
	     if (status_update_reg)
			  next[RECEIVE_COMPLETE] = 1'b1;
	     else
	         next[SEND_TO_HOST] = 1'b1;
	  end 	     

	  if (state[RECEIVE_COMPLETE]) 
	  begin
	     if (!status_update_reg)
			  next[UART_IDLE] = 1'b1;
	     else
	         next[RECEIVE_COMPLETE] = 1'b1;
	  end 	     

	  if (state[WRITE_REGISTER])
	  begin
	     if (transmit_data_ready)
	       next[LOAD_SHIFT_REG] = 1'b1;
	     else 
	       next[WRITE_REGISTER] = 1'b1;
	  end

	if (state[LOAD_SHIFT_REG])
	       next[DELAY_WRITE_DATA_BYTE] = 1'b1;

	if (state[DELAY_WRITE_DATA_BYTE])
	  begin
	     if (!tx_empty)
	       next[WRITE_DATA_BYTE] = 1'b1;
	     else
	       next[DELAY_WRITE_DATA_BYTE] = 1'b1;
	  end

	if (state[WRITE_DATA_BYTE])
	  begin
	     if (tx_empty)
	       next[WRITE_DATA_COMPLETE] = 1'b1;
	     else
	       next[WRITE_DATA_BYTE] = 1'b1;
	  end

	if (state[WRITE_DATA_COMPLETE])
	  begin
	     if (!transmit_data_ready)
	       next[UART_IDLE] = 1'b1;
	     else
	       next[WRITE_DATA_COMPLETE] = 1'b1;
	  end

	
`ifdef SIM
	if (state == (1 << UART_IDLE))
	  state_name = "UART_IDLE";
	else if (state == (1 << RECEIVE_DATA))
	  state_name = "RECEIVE_DATA";
	else if (state == (1 << SEND_TO_HOST))
	  state_name = "SEND_TO_HOST";
	else if (state == (1 << RECEIVE_COMPLETE))
	  state_name = "RECEIVE_COMPLETE";
	else if (state == (1 << STORE_BYTE))
	  state_name = "STORE_BYTE";
	else if (state == (1 << WRITE_REGISTER))
	  state_name = "WRITE_REGISTER";
	else if (state == (1 << LOAD_SHIFT_REG))
	  state_name = "LOAD_SHIFT_REG";
	else if (state == (1 << DELAY_WRITE_DATA_BYTE))
	  state_name = "DELAY_WRITE_DATA_BYTE";
	else if (state == (1 << WRITE_DATA_BYTE))
	  state_name = "WRITE_DATA_BYTE";
	else if (state == (1 << WRITE_DATA_COMPLETE))
	  state_name = "WRITE_DATA_COMPLETE";
`endif
     end // always @ (...
   
//==============================================================================
   // UART Transfer
//	Instantiate the 16x Clock Counter
//==============================================================================
   always @(posedge CLK_50MHZ or negedge RST_N)
   begin
	if (!RST_N)
 	  counter_reg <= 0;
 	else
 	  begin
 	     if ( counter_reg == COUNT_TO_DECIMAL_1 )
 	       counter_reg <= 0;
 	     else
 	       counter_reg <= counter_reg + 1;
 	  end
   end // always @ (posedge CLK_50MHZ or negedge RST_N)

   //----------------------------------------------
   // UART Transfer
   // Instantiate the UART_top Leaf FSM Modules
   //----------------------------------------------
   uart_top uart_inst
     (
      .CLK                    (CLK_50MHZ),
      .RST_N                  (RST_N),
      .DOUT                   (rx_data),
      .DATA_READY             (rx_byte_present),
      .RXD                    (UART_IN),
      .CLK16X                 (clk16x),
      .RDN                    (rx_byte_read),
      .DIN                    (tx_data_reg),
      .TX_BUFFER_EMPTY        (tx_empty),
      .WRN                    (write_tx_byte),
      .SDO                    (UART_OUT)
      );

endmodule

