//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2018
//#
//#
//# File Name:  uart_interface.v
//# Author:     R. Jolly
//# Date:       April 7, 2018
//# Revision:   A
//#
//# Development: Command Decode for BLE Driver 
//# Application: Altera MAX10 FPGA
//# Description: This file contains verilog code which will allow access
//#              to an eight bit bus in the FPGA. The FPGA receives its commands via
//#				 USB.
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			04/7/18 	A			Created			RJJ
//#
//#									
//#
//#######################################################################

`include "../src/define.v"

`timescale 1 ns / 10 ps

module uart_interface
  (
	input wire                 CLK,
	input wire                 RST_N,

  
	output wire                UART_OUT, //To external 
    input wire                 UART_IN, //From external
  
	output  reg  [7:0]          MEMORY_ADDRESS,
	output  reg  [7:0]          MEMORY_DATA,
	output  reg                 MEMORY_ENABLE,
	input   wire                MEMORY_READY,
	output  reg                 MEMORY_OUTPUT_ENABLE,
	output  reg                 MEMORY_REQUEST,
	input   wire                MEMORY_GRANT,
	output  wire                MEMORY_END_OF_MESSAGE,
	
	
	input   wire [7:0]          TRANSMIT_DATA,
	input   wire                TRANSMIT_ENABLE,
	output  reg                 TRANSMIT_READY,
	input   wire [7:0]          TRANSMIT_COUNT,
	input   wire                TRANSMIT_START,
	
	output  wire [7:0]          TEST_OUT1,
	output  wire [7:0]          TEST_OUT2	
	


	);



   //----------------------------------------------
   // Parameter Declarations                        
   //----------------------------------------------
   //Memory Interface State Machine
   parameter IDLE                   = 0,
	     RECEIVE_DATA               = 1,
		 STORE_BYTE                 = 2,
		 BUS_REQUEST                = 3,
		 BUS_GRANT                  = 4,
		 SEND_ENABLE                = 5,
		 WAIT_FOR_READY             = 6,
		 COMPLETE_RD_CYCLE          = 7;

   //Transmit State Machine
   parameter TRANSMIT_IDLE          = 0,
	     ENABLE_RECEIVED            = 1,
		 READ_BYTE                  = 2,
		 ASSERT_READY               = 3,
		 TRANSMIT_BYTE              = 4,
		 COMPLETE_CYCLE             = 5;
		 
`ifdef FAST_UART
		parameter COUNT_TO_DECIMAL       = 4;
		parameter COUNT_TO_DECIMAL_3     = 8'h36;
		parameter COUNT_TO_DECIMAL_2     = 8'h23; // 66 MHz Main Clock; 115200 UART Baud Rate
		parameter COUNT_TO_DECIMAL_1     = 8'h19; // 50 MHz Main Clock; 115200 UART Baud Rate  
`else
//		parameter COUNT_TO_DECIMAL       = 54;
		parameter COUNT_TO_DECIMAL_4     = 8'h6d;
		parameter COUNT_TO_DECIMAL_3     = 8'h36;
		parameter COUNT_TO_DECIMAL_2     = 8'h23; // 66 MHz Main Clock; 115200 UART Baud Rate
		parameter COUNT_TO_DECIMAL_1     = 8'h1b; // 50 MHz Main Clock; 115200 UART Baud Rate
		parameter COUNT_TO_DECIMAL       = 4;
`endif

   //------------------------------------------------
   // Wire and Register Declarations                
   //------------------------------------------------
   	reg [7:0]                  memory_state, memory_next;
   	reg [5:0]                  transmit_state, transmit_next;

    // Bit Count Registers
	//reg [3:0]                  in_bit_count;
	//reg [3:0]                  out_bit_count;
	
	
	// Storage Registers
	reg [7:0]                  rcv_storage_reg;
	reg [2:0]                  store_reg_state; 
	
	
	//Metastable Registers
	reg                        uart_in_reg;
	reg                        usb_rxf_reg;
	reg                        usb_wr_reg;
	
	//write_tx_byte_reg for signalling the UART transmitter
	//to start transmitting
	reg                        write_tx_byte_reg;

	//STATE_OUT Regisetrs
    reg [3:0]                  STATE_OUT;

	//UART Interface signals
	wire [7:0]                  rx_data;
    wire                        rx_byte_present;
    wire                        UART_SERIAL_RX;
    wire                        en_16_x_baud;
    reg                         rx_byte_read;
	wire [7:0]                  tx_data_reg;
    wire                        tx_empty;
    reg                         write_tx_byte;
	
	//Automated UART write
	
	//Transmit registers
    reg  [7:0]                  transmit_byte;
    reg  [7:0]                  memory_address_counter;
	
	//Receive registers
    reg  [7:0]                  memory_storage_byte;
	
	//End of message registers
    reg                         end_of_message_flag;
	
	//TEST ONLY
	wire [7:0]                 test_bus1;
	
   	
`ifdef SIM
   reg [8*26:1] memory_state_name;
   reg [8*26:1] transmit_state_name;
`endif

   //------------------------------------------------
   // Signal Assignments  
   //------------------------------------------------

`ifdef FAST_UART
	reg	[7:0]			counter_reg;
//	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_3 - 1) );
	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_1 - 1) );
`else
	reg	[7:0]			counter_reg;
//	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_3 - 1) );
	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_1 - 1) );
`endif
	
    //UART Transmission byte
	assign   tx_data_reg = transmit_byte;
	
	//End of Message register
	assign   MEMORY_END_OF_MESSAGE = end_of_message_flag;

	assign          TEST_OUT1[0] = rx_data[0];
	assign          TEST_OUT1[1] = rx_data[1];//endpoint_busy;
	assign          TEST_OUT1[2] = rx_data[2];//endpoint_busy;
	assign          TEST_OUT1[3] = rx_data[3];
	assign          TEST_OUT1[4] = rx_data[4];
	assign          TEST_OUT1[5] = rx_data[5];
	assign          TEST_OUT1[6] = rx_data[6];
	assign          TEST_OUT1[7] = rx_data[7];
	
	assign          TEST_OUT2 = test_bus1;
	
/*	assign          TEST_OUT2[0] = STATE_OUT[0];
	assign          TEST_OUT2[1] = STATE_OUT[1];
	assign          TEST_OUT2[2] = STATE_OUT[2];
	assign          TEST_OUT2[3] = STATE_OUT[3];
	assign          TEST_OUT2[4] = MEMORY_REQUEST;
	assign          TEST_OUT2[5] = MEMORY_GRANT;
	assign          TEST_OUT2[6] = MEMORY_ENABLE;
	assign          TEST_OUT2[7] = MEMORY_READY;
*/	
   //-----------------------------------------------
   // Create a State Test Output Signal
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  STATE_OUT <= 0;
	else
	  begin
	     if (memory_state[IDLE])
           STATE_OUT <= 4'h0;
	     if (memory_state[RECEIVE_DATA])
	       STATE_OUT <= 4'h1;
	     if (memory_state[STORE_BYTE])
	       STATE_OUT <= 4'h2;
	     if (memory_state[BUS_REQUEST])
	       STATE_OUT <= 4'h3;
	     if (memory_state[BUS_GRANT])
	       STATE_OUT <= 4'h4;
	     if (memory_state[SEND_ENABLE])
	       STATE_OUT <= 4'h5;
	     if (memory_state[WAIT_FOR_READY])
	       STATE_OUT <= 4'h6;
	     if (memory_state[COMPLETE_RD_CYCLE])
	       STATE_OUT <= 4'h7;
	  end // else: !if(!RST_N)
     end // always @ (posedge CLK or negedge RST_N)


   //-----------------------------------------------
   // Receive: Store byte from UART decoder.  
   // memory_state[RECEIVE_DATA]. Indicates to User Code
   // that a byte is ready to be read from the Receive
   // Unit.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	     memory_storage_byte <= 0;
		 rx_byte_read <= 1'b1;
    end
	else
	  begin
	     if(memory_state[RECEIVE_DATA])
		 begin
	         memory_storage_byte <= rx_data;
			 rx_byte_read <= 1'b0;
	     end
		 else if(memory_state[STORE_BYTE])
		 begin
			 rx_byte_read <= 1'b1;
	     end
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Receive: Send Bus Request to Memory Storage Unit.  
   // The Bus Request/Grant mechanism will allow only 
   // one device to access the Memory_mux unit and 
   // avoid collisions. 
   // 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	     MEMORY_REQUEST <= 1'b0;
	else
	  begin
	     if(memory_state[BUS_REQUEST])
	         MEMORY_REQUEST <= 1'b1;
		 else
		     MEMORY_REQUEST <= 1'b0;
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Receive: Send Enable to Memory Storage Unit.  
   // The Enable informs the memory_mux unit that
   // UART_interface is ready to transmit its byte
   // to be stored in memory.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	     MEMORY_ENABLE <= 1'b0;
		 MEMORY_ADDRESS <= 0;
		 MEMORY_DATA <= 0;
	end
	else
	  begin
	     if(memory_state[SEND_ENABLE])
		 begin
		     MEMORY_DATA <= memory_storage_byte;
			 MEMORY_ADDRESS <= memory_address_counter;
	         MEMORY_ENABLE <= 1'b1;
		 end
		 else if(MEMORY_READY)
		     MEMORY_ENABLE <= 1'b0;
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Receive: Update the address for the location  
   // of the received byte to store in memory_mux.
   // 
   // 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	     memory_address_counter <= 0;
	end
	else
	  begin
		 if(memory_state[COMPLETE_RD_CYCLE] & end_of_message_flag)
		     memory_address_counter <= 0;
	     else if(memory_state[COMPLETE_RD_CYCLE])
		 begin
		     memory_address_counter <= memory_address_counter + 1'd1;
		 end
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Receive: Read each received byte from the UART 
   // unit and determine if it is the end of message.
   // 
   // 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	     end_of_message_flag <= 1'b0;
	end
	else
	  begin
	     if(memory_state[RECEIVE_DATA])
		 begin
		    case(rx_data)
			`END_OF_MESSAGE:
			   end_of_message_flag <= 1'b1;
			default:
			   end_of_message_flag <= 1'b0;
			endcase
		     
		 end
		 else if(memory_state[COMPLETE_RD_CYCLE])
		     end_of_message_flag <= 1'b0;
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Transmit: Accept byte from memory_mux for trans-  
   // mission via UART.
   // 
   // 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
   begin
	if (!RST_N)
	begin
	     transmit_byte <= 0;
	end
	else
	  begin
	     if(transmit_state[READ_BYTE])
		 begin
		     transmit_byte <= TRANSMIT_DATA;
		 end
	   end
   end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Transmit: Assert Transmit Ready  
   // 
   // 
   // 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	     TRANSMIT_READY <= 1'b0;
	end
	else
	  begin
	     if(transmit_state[ASSERT_READY])
		 begin
		     TRANSMIT_READY <= 1'b1;
		 end
		 else
		 begin
		     TRANSMIT_READY <= 1'b0;
		 end
	  end	 
     end // always @ (posedge CLK or negedge RST_N)


   //--------------------------------------------------
   // UART-Memory State Machine
   //
   //   parameter IDLE               = 0,
   //     RECEIVE_DATA               = 1,
   //     STORE_BYTE                 = 2,
   //     BUS_REQUEST                = 3,
   //     BUS_GRANT                  = 4,
   //     SEND_ENABLE                = 5,
   //     WAIT_FOR_READY             = 6,
   //     COMPLETE_RD_CYCLE          = 7;
   //
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     memory_state <= 12'h00;
	     memory_state[IDLE] <= 1'b1;
	  end
	else
	  memory_state <= memory_next;
     end

   always @ ( /*AUTOSENSE*/memory_state or tx_empty
		 or rx_byte_present or MEMORY_GRANT or MEMORY_READY
		 )
     begin
	memory_next = 12'h00;

	if (memory_state[IDLE])
	begin
        if (rx_byte_present)
			memory_next[RECEIVE_DATA] = 1'b1;
		else
			memory_next[IDLE] = 1'b1;
	end

	if (memory_state[RECEIVE_DATA])
	       memory_next[STORE_BYTE] = 1'b1;

	  if(memory_state[STORE_BYTE]) 
	  begin
	     memory_next[BUS_REQUEST] = 1'b1;
	  end 	     

	if (memory_state[BUS_REQUEST]) 
	  begin
	     if (MEMORY_GRANT)
			  memory_next[BUS_GRANT] = 1'b1;
	     else
	         memory_next[BUS_REQUEST] = 1'b1;
	  end 	     

	  if (memory_state[BUS_GRANT]) 
	  begin
			  memory_next[SEND_ENABLE] = 1'b1;
	  end 	     

	  if (memory_state[SEND_ENABLE])
	  begin
	       memory_next[WAIT_FOR_READY] = 1'b1;
	  end

	if (memory_state[WAIT_FOR_READY])
	     if (MEMORY_READY)
	       memory_next[COMPLETE_RD_CYCLE] = 1'b1;
	     else 
	       memory_next[WAIT_FOR_READY] = 1'b1;

	if (memory_state[COMPLETE_RD_CYCLE])
    begin
	     if (!MEMORY_READY)
	       memory_next[IDLE] = 1'b1;
	     else 
	       memory_next[COMPLETE_RD_CYCLE] = 1'b1;

    end


`ifdef SIM
	if (memory_state == (1 << IDLE))
	  memory_state_name = "IDLE";
	else if (memory_state == (1 << RECEIVE_DATA))
	  memory_state_name = "RECEIVE_DATA";
	else if (memory_state == (1 << STORE_BYTE))
	  memory_state_name = "STORE_BYTE";
	else if (memory_state == (1 << BUS_REQUEST))
	  memory_state_name = "BUS_REQUEST";
	else if (memory_state == (1 << BUS_GRANT))
	  memory_state_name = "BUS_GRANT";
	else if (memory_state == (1 << SEND_ENABLE))
	  memory_state_name = "SEND_ENABLE";
	else if (memory_state == (1 << WAIT_FOR_READY))
	  memory_state_name = "WAIT_FOR_READY";
	else if (memory_state == (1 << COMPLETE_RD_CYCLE))
	  memory_state_name = "COMPLETE_RD_CYCLE";
`endif
     end // always @ (...

   //--------------------------------------------------
   // Transmit State Machine
   //
   //parameter TRANSMIT_IDLE          = 0,
   //	       ENABLE_RECEIVED        = 1,
   //	       READ_BYTE              = 2,
   //	       ASSERT_READY           = 3,
   //	       TRANSMIT_BYTE          = 4,
   //	       COMPLETE_CYCLE         = 5;
   //
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     transmit_state <= 12'h00;
	     transmit_state[TRANSMIT_IDLE] <= 1'b1;
	  end
	else
	  transmit_state <= transmit_next;
     end

   always @ ( /*AUTOSENSE*/transmit_state or TRANSMIT_ENABLE
		 or TRANSMIT_READY or tx_empty
		 )
     begin
	transmit_next = 12'h00;

	if (transmit_state[TRANSMIT_IDLE])
	begin
        if (TRANSMIT_ENABLE)
			transmit_next[ENABLE_RECEIVED] = 1'b1;
		else
			transmit_next[TRANSMIT_IDLE] = 1'b1;
	end

	if (transmit_state[ENABLE_RECEIVED])
	       transmit_next[READ_BYTE] = 1'b1;

	  if(transmit_state[READ_BYTE]) 
	       transmit_next[ASSERT_READY] = 1'b1;

	if (transmit_state[ASSERT_READY]) 
	  begin
	     if (TRANSMIT_READY)
			  transmit_next[TRANSMIT_BYTE] = 1'b1;
	     else
	         transmit_next[ASSERT_READY] = 1'b1;
	  end 	     

	  if (transmit_state[TRANSMIT_BYTE]) 
	  begin
	     if (tx_empty)
			  transmit_next[SEND_ENABLE] = 1'b1;
	     else 
	       transmit_next[TRANSMIT_BYTE] = 1'b1;
	  end 	     

	if (transmit_state[COMPLETE_CYCLE])
    begin
	     if (!tx_empty)
	       transmit_next[TRANSMIT_IDLE] = 1'b1;
	     else 
	       transmit_next[COMPLETE_CYCLE] = 1'b1;

    end

`ifdef SIM
	if (transmit_state == (1 << TRANSMIT_IDLE))
	  transmit_state_name = "TRANSMIT_IDLE";
	else if (transmit_state == (1 << ENABLE_RECEIVED))
	  transmit_state_name = "ENABLE_RECEIVED";
	else if (transmit_state == (1 << READ_BYTE))
	  transmit_state_name = "READ_BYTE";
	else if (transmit_state == (1 << ASSERT_READY))
	  transmit_state_name = "ASSERT_READY";
	else if (transmit_state == (1 << TRANSMIT_BYTE))
	  transmit_state_name = "TRANSMIT_BYTE";
	else if (transmit_state == (1 << COMPLETE_CYCLE))
	  transmit_state_name = "COMPLETE_CYCLE";
`endif
     end // always @ (...
	 
//==============================================================================
//	Instantiate the 16x Clock Counter
//==============================================================================
`ifdef FAST_UART
   // For Simulation use a much faster clock to speed up test
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
 	  counter_reg <= 0;
 	else
 	  begin
 	     if ( counter_reg == COUNT_TO_DECIMAL_1 )
 	       counter_reg <= 0;
 	     else
 	       counter_reg <= counter_reg + 1;
 	  end
     end // always @ (posedge CLK or negedge RST_N)
`else
   // For Simulation use a much faster clock to speed up test
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
 	  counter_reg <= 0;
 	else
 	  begin
 	     if ( counter_reg == COUNT_TO_DECIMAL_1 )
 	       counter_reg <= 0;
 	     else
 	       counter_reg <= counter_reg + 1;
 	  end
     end // always @ (posedge CLK or negedge RST_N)
`endif

   //----------------------------------------------
   // Instantiate the UART_top Leaf FSM Modules
   //----------------------------------------------
   uart_top                   UART_TOP_INST
     (
      .CLK                    (CLK),
      .RST_N                  (RST_N),
      .DOUT                   (rx_data),
      .DATA_READY             (rx_byte_present),
      .RXD                    (UART_IN),
      .CLK16X                 (clk16x),
      .RDN                    (rx_byte_read),
      .DIN                    (tx_data_reg),
      .TX_BUFFER_EMPTY        (tx_empty),
      .WRN                    (write_tx_byte),
      .SDO                    (UART_OUT),
	  
	  .TEST_OUT               (test_bus1)
      );

endmodule // uart_slave

