//================================================================
//
//	Smartronix Proprietary Information
//
//
//	File Name:	serial_uart_comms_model.v
//	Author:	R. Jolly
//	Date:		November 24, 2008
//	Revision:	A
//
//
//	Development:	Environment Control Module UAV
//	Application:	Altera Stratix II
//	Description:	This file contains Verilog code that implements the serial
//				UART Communications. Used only during Modelsim simulations.
//				It has dependent files UART_iface.v
//
//
//
//	Revision History:
//
//	DATE		VERSION		DETAILS
//	11/24/08		A		Created by R Jolly
//
//
//
//
//
//
//
//
//
//
//
//==================================================================

`include "../Testbench/tb_define.v"


module serial_uart_comms_model (		
        CLK,
		RESET,

		UART_RXD_IN,
		UART_TXD_OUT,

		CONTROL_REGISTER,
		STATUS_REGISTER,
		RECEIVE_REGISTER,
		SEND_REGISTER

		);

//==================================================================
// Parameter Declarations
//==================================================================
	parameter IDLE					= 0,
		WAIT_FOR_CMD				= 1,
		CMD_TX_WRITE				= 2,
		WRITE_CMD_NOP				= 3,
		WR_CMD_WAIT_BUF_EMPTY			= 4,
		DATA_TX_WRITE				= 5,
		WRITE_DATA_NOP				= 6,
		WR_DATA_WAIT_BUF_EMPTY			= 7,
		RECEIVE_BYTE				= 8;

`ifdef FAST_UART
		parameter COUNT_TO_DECIMAL       = 4;
		parameter COUNT_TO_DECIMAL_3     = 8'h36;
		parameter COUNT_TO_DECIMAL_2     = 8'h23; // 66 MHz
		parameter COUNT_TO_DECIMAL_1     = 8'h19; // 50 MHz Main Clock; 115200 UART Baud Rate  
`else
//		parameter COUNT_TO_DECIMAL       = 54;
		parameter COUNT_TO_DECIMAL_4     = 8'h6d;
		parameter COUNT_TO_DECIMAL_3     = 8'h36;
		parameter COUNT_TO_DECIMAL_2     = 8'h23;
		parameter COUNT_TO_DECIMAL_1     = 8'h1b; // 50 MHz Main Clock; 115200 UART Baud Rate
		parameter COUNT_TO_DECIMAL       = 4;
`endif

   
//==================================================================
// IO Port Declarations
//==================================================================
	input						CLK;
	input						RESET;

	input                       UART_RXD_IN;
	output                      UART_TXD_OUT;

	output  [7:0]               CONTROL_REGISTER;
	input   [7:0]               STATUS_REGISTER;
	output	[7:0]               RECEIVE_REGISTER;
	input   [7:0]               SEND_REGISTER;

//==================================================================
// Register Declarations
//==================================================================
	wire  [7:0]                 RECEIVE_REGISTER;
  
  

	// CONTROL Register
	reg   [7:0]                 CONTROL_REGISTER;
	reg                         control_update_reg;
	reg                         status_update_reg;

	// State Machine Registers
	reg [11:0]					state, next;


   	reg [7:0]					tx_data_reg;


   	reg						write_tx_byte;
   	reg [2:0]					tx_byte_count;
   	reg [2:0]					total_tx_bytes;
   	reg [8:0]					bytes_written_reg;
//   	reg [3:0]					STATE_OUT;
   
   	wire						tx_buffer_full;
   	wire						tx_buffer_empty;

	// Transmit Registers
	reg						transmit_data_ready;
	reg						transmit_data_ready_reg;
	reg						cmd_reg;

	// Receive Registers
	reg						receive_data_ready;
	reg						receive_data_ready_reg;


	// uart_iface.v registers
	wire	[7:0]					uart_data_out;
	wire						uart_data_ready;
	wire						UART_RXD_IN;
	wire						clk16x;
	reg						uart_rdn;
	wire	[7:0]					uart_data_in;
	wire						uart_tx_buffer_empty;
	reg						uart_write_enable;
	wire						UART_TXD_OUT;

	// UART Clocking 
	//wire	[8:0]					counter_output;

	// uart_rdn_reg 
	reg						uart_rdn_reg ;



`ifdef SIM
   reg [8*25:1] state_name;
`endif
   
//==================================================================
// Signal Assignments  
//==================================================================
	assign 	 tx_buffer_full = !tx_buffer_empty;

	assign	RECEIVE_REGISTER = uart_data_out;

`ifdef FAST_UART
	reg	[7:0]			counter_reg;
//	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_3 - 1) );
	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_1 - 1) );
`else
	reg	[7:0]			counter_reg;
//	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_3 - 1) );
	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_1 - 1) );
`endif
	

	assign	uart_data_in 	= tx_data_reg;

//==================================================================
// Create a Tx State Test Output Signal
//==================================================================
//   always @(posedge CLK or negedge RESET)
//     begin
//	if (!RESET)
//	  STATE_OUT <= 0;
//	else
//	  begin
//	     if (state[IDLE])
//               STATE_OUT <= 4'h0;
//	     if (state[WAIT_FOR_CMD])
//	       STATE_OUT <= 4'h1 ;
//	     if (state[CMD_TX_WRITE])
//	       STATE_OUT <= 4'h2 ;
//	     if (state[WRITE_CMD_NOP])
//	       STATE_OUT <= 4'h3 ;
//	     if (state[WR_CMD_WAIT_BUF_EMPTY])
//	       STATE_OUT <= 4'h4 ;
//	     if (state[DATA_TX_WRITE])
//	       STATE_OUT <= 4'h5 ;
//	     if (state[WRITE_DATA_NOP])
//	       STATE_OUT <= 4'h6 ;
//	  end // else: !if(!RST_N)
//     end // always @ (posedge CLK or negedge RST_N)

//==============================================================================
//	Instantiate the 16x Clock Counter
//==============================================================================
`ifdef FAST_UART
   // For Simulation use a much faster clock to speed up test
   always @(posedge CLK or negedge RESET)
     begin
	if (!RESET)
 	  counter_reg <= 0;
 	else
 	  begin
 	     if ( counter_reg == COUNT_TO_DECIMAL_1 )
 	       counter_reg <= 0;
 	     else
 	       counter_reg <= counter_reg + 1;
 	  end
     end // always @ (posedge CLK or negedge RESET)
`else
   // For Simulation use a much faster clock to speed up test
   always @(posedge CLK or negedge RESET)
     begin
	if (!RESET)
 	  counter_reg <= 0;
 	else
 	  begin
 	     if ( counter_reg == COUNT_TO_DECIMAL_1 )
 	       counter_reg <= 0;
 	     else
 	       counter_reg <= counter_reg + 1;
 	  end
     end // always @ (posedge CLK or negedge RESET)
`endif


//=================================================================================
//	Receive Cycle: uart_rdn, this register informs the uart_receiver.v that the  
//	receive word has been acknowledged and transferred to serial_uarts_comms.v 
//=================================================================================


	always @( negedge CLK or negedge RESET )
	begin
		if ( !RESET )
		begin
			uart_rdn <= 1'b1;
			uart_rdn_reg <= 1'b0;
		end
		else
		begin
			if ( state[RECEIVE_BYTE] & !uart_rdn_reg )
			begin
				uart_rdn <= 1'b0;
				uart_rdn_reg <= 1'b1;
			end
			else if ( state[IDLE] )
			begin
				uart_rdn <= 1'b1;
				uart_rdn_reg <= 1'b0;
			end
		end
	end



//=================================================================================
//	Receive Cycle: CONTROL_REGISTER, this register informs the ARM9 when sensor
//	data has been updated. uart_rdn
//=================================================================================


	always @( negedge CLK or negedge RESET )
	begin
		if ( !RESET )
		begin
			CONTROL_REGISTER <= 8'b0;
			control_update_reg <= 1'b0;
			status_update_reg <= 1'b0;
		end
		else
		begin
			if( STATUS_REGISTER[3] && !status_update_reg )
			begin
				CONTROL_REGISTER[3] <= 1'b0;
				status_update_reg <= 1'b1;
			end
			else if( STATUS_REGISTER[3]&& status_update_reg )
			begin
				status_update_reg <= 1'b1;
			end
			else if( !STATUS_REGISTER[3] && status_update_reg )
			begin
				status_update_reg <= 1'b0;
			end

			if ( state[RECEIVE_BYTE] & !control_update_reg )
			begin
				CONTROL_REGISTER[3] <= CONTROL_REGISTER[3] || 1'b1;
				control_update_reg <= 1'b1;
			end
			else if ( state[IDLE] )
			begin
				control_update_reg <= 1'b0;
			end
		end
	end



//==================================================================
// Receive: Create the receive data from UART signal using the uart_data_ready
//==================================================================
	always @(posedge CLK or negedge RESET)
	begin
		if (!RESET)
		begin
			receive_data_ready <= 1'b0;
			receive_data_ready_reg <= 1'b0;
		end
		else 
	  	begin
	     		if ( uart_data_ready & !receive_data_ready_reg )
			begin
				receive_data_ready <= 1'b1;
	       		receive_data_ready_reg <= 1'b1;
			end
	     		else if ( uart_data_ready & receive_data_ready_reg )
			begin
				receive_data_ready <= 1'b0;
	       		receive_data_ready_reg <= 1'b1;
			end
	     		else if ( state[IDLE] )
			begin
				receive_data_ready <= 1'b0;
	       		receive_data_ready_reg <= 1'b0;
			end
	  	end
     end // always @ (posedge CLK or negedge RESET)   
      
//==================================================================
// Transmit: transmit_data_ready will be equated with STATUS_REGISTER[1]
//==================================================================
	always @(posedge CLK or negedge RESET)
	begin
		if (!RESET)
		begin
			transmit_data_ready <= 0;
			transmit_data_ready_reg <= 0;
		end
		else 
		begin
			if ( STATUS_REGISTER[1] && !transmit_data_ready_reg )
			begin
				transmit_data_ready <= 1'b1;
				transmit_data_ready_reg <= 1'b1;
			end
			else if ( STATUS_REGISTER[1] && transmit_data_ready_reg )
			begin
				transmit_data_ready <= 1'b0;
				transmit_data_ready_reg <= 1'b1;
			end
			else if ( state[IDLE] )
			begin
				transmit_data_ready <= 0;
				transmit_data_ready_reg <= 0;
			end
		end		
	end // always @ (posedge CLK or negedge RESET)

//==================================================================
// Transmit: If UART is busy sending previous byte then
// set bit: STATUS_REGISTER[2]
//==================================================================
	always @(posedge CLK or negedge RESET)
	begin
		if (!RESET)
		begin
			CONTROL_REGISTER[2] <= 0;
		end
		else 
		begin
			if ( uart_tx_buffer_empty )
			begin
				CONTROL_REGISTER[2] <= 1'b1;
			end
			else
			begin
				CONTROL_REGISTER[2] <= 1'b0;
			end
		end		
	end // always @ (posedge CLK or negedge RESET)
   
//==================================================================
// Transmit: Create a tx_byte_count transmit_data_ready 
//==================================================================
   always @(posedge CLK or negedge RESET)
     begin
	if (!RESET)
	  tx_byte_count <= 0;
	else 
	  begin
	     if (state[DATA_TX_WRITE] && clk16x)
	       tx_byte_count <= tx_byte_count + 1;
	     else if (state[WAIT_FOR_CMD])
	       tx_byte_count <= 0;
	  end
     end // always @ (posedge CLK or negedge RESET)
   
//==================================================================
// Transmit: Create an total_tx_bytes signal 
//==================================================================
   always @(posedge CLK or negedge RESET)
     begin
	if (!RESET)
	  total_tx_bytes <= 0;
	else 
	  begin
	     if (state[CMD_TX_WRITE])
	       total_tx_bytes <= (`TX_PAYLOAD_BYTES - 1);
	     else if (state[WAIT_FOR_CMD])
	       total_tx_bytes <= 0;
	  end
     end // always @ (posedge CLK or negedge RESET)      


//==================================================================
// Transmit: Create a uart_write_enable Signal 
//==================================================================
	always @(posedge CLK or negedge RESET)
	begin
		if (!RESET)
			uart_write_enable <= 1;
		else 
		begin
			if ( state[DATA_TX_WRITE] )
				uart_write_enable <= 0;
			//else if ( state[IDLE] )
			else 
				uart_write_enable <= 1;
		end	
	end // always @ (posedge CLK or negedge RESET)   
   

   
//==================================================================
// Transmit: Command Stuff
//==================================================================
	always @(posedge CLK or negedge RESET)
	begin
		if (!RESET)
			tx_data_reg <= 0;
		else
		begin
			if( transmit_data_ready )
			begin
				tx_data_reg <= SEND_REGISTER;
			end
			else 
			begin
				tx_data_reg <= tx_data_reg;
			end
		end 
	end // always @ (posedge CLK or negedge RESET)
   
//==================================================================
// Finite  State Machine
//==================================================================
   // Next State Logic
   always @(posedge CLK or negedge RESET)
     begin
	if (!RESET)
	  begin
	     state <= 12'h000;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

     // State Definitions
   always @ ( state or receive_data_ready or transmit_data_ready 
	     or tx_buffer_full or tx_byte_count or uart_data_ready 
		or uart_write_enable or clk16x or /*counter_reg or */status_update_reg )
     begin
	next = 12'h000;
	
	if (state[IDLE])
	begin
		if ( receive_data_ready )
			next[RECEIVE_BYTE] = 1'b1;
		else if( transmit_data_ready )
			next[DATA_TX_WRITE] = 1'b1;
		else 
			next[IDLE] = 1'b1;
	end


	if (state[DATA_TX_WRITE])
	  begin
	     if (clk16x)
	       next[WRITE_DATA_NOP] = 1'b1;
	     else
	       next[DATA_TX_WRITE] = 1'b1;
	  end
	
	if (state[WRITE_DATA_NOP])
	  next[WR_DATA_WAIT_BUF_EMPTY] = 1'b1;

	if (state[WR_DATA_WAIT_BUF_EMPTY])
	  begin
	     //if (!tx_buffer_full)
	    //   begin
		  if ( !uart_tx_buffer_empty )
		    next[WR_DATA_WAIT_BUF_EMPTY] = 1'b1;
		  else
		    next[IDLE] = 1'b1;
	       //end
	     //else
	     //  next[IDLE] = 1'b1;
	  end 

	if (state[RECEIVE_BYTE])
	  begin
	     if ( !uart_rdn )
	       next[IDLE] = 1'b1;
	     else
	       next[RECEIVE_BYTE] = 1'b1;
	  end

//==================================================================
// For simulation, display the state names
//==================================================================
`ifdef SIM
	if (state == (1 << IDLE))
	  state_name = "IDLE";
	else if (state == (1 << WAIT_FOR_CMD))
	  state_name = "WAIT_FOR_CMD";
	else if (state == (1 << CMD_TX_WRITE))
	  state_name = "CMD_TX_WRITE";
	else if (state == (1 << WRITE_CMD_NOP))
	  state_name = "WRITE_CMD_NOP";
	else if (state == (1 << WR_CMD_WAIT_BUF_EMPTY))
	  state_name = "WR_CMD_WAIT_BUF_EMPTY";
	else if (state == (1 << DATA_TX_WRITE))
	  state_name = "DATA_TX_WRITE";
	else if (state == (1 << WRITE_DATA_NOP))
	  state_name = "WRITE_DATA_NOP";
	else if (state == (1 << WR_DATA_WAIT_BUF_EMPTY))
	  state_name = "WR_DATA_WAIT_BUF_EMPTY";
	else if (state == (1 << RECEIVE_BYTE))
	  state_name = "RECEIVE_BYTE";
`endif
     end // always @ (...
   
//==============================================================================
//	Instantiate the UART Interface
//==============================================================================

	uart_iface					uart_iface_inst
	(
		.CLK                            (CLK),
		.RST_N                          (RESET),

		.DOUT                           (uart_data_out),
		.DATA_READY                     (uart_data_ready),

		.RXD                            (UART_RXD_IN),
		.CLK16X                         (clk16x),
		.RDN                            (uart_rdn),
		.DIN                            (uart_data_in),

		.TX_BUFFER_EMPTY                (uart_tx_buffer_empty),
		.WRN                            (uart_write_enable),
		.SDO                            (UART_TXD_OUT)
		);


//==============================================================================
//	Instantiate the 16x Clock Counter
//==============================================================================
`ifdef FAST_UART
   // For Simulation use a much faster clock to speed up test
   always @(posedge CLK or negedge RESET)
     begin
	if (!RESET)
 	  counter_reg <= 0;
 	else
 	  begin
 	     if ( counter_reg == COUNT_TO_DECIMAL_2 )
 	       counter_reg <= 0;
 	     else
 	       counter_reg <= counter_reg + 1;
 	  end
     end // always @ (posedge CLK or negedge RST_N)
`else
   counter_53 counter_inst 
     (
      .Q                      (counter_output),
      .CLK                    (CLK),
      .SINIT                  (!RST_N)
      );
`endif


endmodule // serial_uart_comms.v


`ifdef SIM
`else
module counter_53 
  (
   Q,
   CLK,
   SINIT
   ); 
   output [8:0] Q;
   input 	CLK;
   input 	SINIT;
endmodule // counter_53

`endif




